import unittest
from bookshelf.api_v2 import vagrant
from fabric.api import run, sudo
from bookshelf.tests.api_v2.vagrant_based_tests import (
    with_ephemeral_vagrant_box,
)


class VagrantBasedTests(unittest.TestCase):
    @with_ephemeral_vagrant_box(
        verbose=True,
        images=['ubuntu/trusty64', 'ubuntu/vivid64'])
    def test_vagrant_module(self, *args, **kwargs):

        vagrant.install_virtualbox(distribution='ubuntu',
                                   force_setup=True)
        self.assertTrue(sudo('dpkg-query -l virtualbox-5.0'
                             '| grep -q ^.i').return_code == 0)

        vagrant.install_vagrant(distribution='ubuntu', version='1.7.4')
        run('vagrant init')
        self.assertTrue(
            'Vagrantfile' in run('ls')
        )

        vagrant.install_vagrant_plugin(plugin='ansible')
        self.assertTrue(
            'ansible' in run('vagrant plugin list')
        )

        self.assertTrue(
            vagrant.is_vagrant_plugin_installed('ansible')
        )


if __name__ == '__main__':
    unittest.main(verbosity=4, failfast=True)



# vim: ai ts=4 sts=4 et sw=4 ft=python fdm=indent et foldlevel=0

from fabric.api import sudo, local, settings, run
from fabric.context_managers import hide
from fabric.contrib.files import (append as file_append,
                                  contains as file_contains)

from bookshelf.api_v2.os_helpers import install_ubuntu_development_tools

from bookshelf.api_v2.logging_helpers import log_green


def add_epel_yum_repository():
    """
    Install a repository that provides epel packages/updates

    usage:
        add_epel_yum_repository()

    """
    yum_install(packages=["epel-release"])


def add_zfs_apt_repository():
    """ adds the ZFS repository """
    with settings(hide('warnings', 'running', 'stdout'),
                  warn_only=False, capture=True):
        sudo('DEBIAN_FRONTEND=noninteractive /usr/bin/apt-get update')
        install_ubuntu_development_tools()
        apt_install(packages=['software-properties-common',
                              'dkms',
                              'linux-headers-generic',
                              'build-essential'])
        sudo('echo | add-apt-repository ppa:zfs-native/stable')
        sudo('DEBIAN_FRONTEND=noninteractive /usr/bin/apt-get update')
        return True


def add_zfs_yum_repository():
    """ adds the yum repository for ZFSonLinux """
    ZFS_REPO_PKG = (
        "http://archive.zfsonlinux.org/epel/zfs-release.el7.noarch.rpm"

    )
    yum_install_from_url('zfs-release', ZFS_REPO_PKG)


def apt_install(**kwargs):
    """
        installs a apt package
    """
    for pkg in list(kwargs['packages']):
        if is_package_installed(distribution='ubuntu', pkg=pkg) is False:
            sudo("DEBIAN_FRONTEND=noninteractive /usr/bin/apt-get install -y %s" % pkg)
        # if we didn't abort above, we should return True
        return True


def apt_install_from_url(pkg_name, url, log=False):
    """ installs a pkg from a url
        p pkg_name: the name of the package to install
        p url: the full URL for the rpm package
    """
    if is_package_installed(distribution='ubuntu', pkg=pkg_name) is False:

        if log:
            log_green(
                "installing %s from %s" % (pkg_name, url))

        with settings(hide('warnings', 'running', 'stdout'),
                      capture=True):

            sudo("wget -c -O %s.deb %s" % (pkg_name, url))
            sudo("dpkg -i %s.deb" % pkg_name)
            # if we didn't abort above, we should return True
            return True


def apt_add_repository_from_apt_string(apt_string, apt_file):
    """ adds a new repository file for apt """

    apt_file_path = '/etc/apt/sources.list.d/%s' % apt_file

    if not file_contains(apt_file_path, apt_string.lower(), use_sudo=True):
        file_append(apt_file_path, apt_string.lower(), use_sudo=True)

        with hide('running', 'stdout'):
            output = sudo("DEBIAN_FRONTEND=noninteractive /usr/bin/apt-get update")
        if 'Some index files failed to download' in output:
            raise SystemExit(1)
        else:
            # if we didn't abort above, we should return True
            return True


def apt_add_key(keyid, keyserver='keyserver.ubuntu.com', log=False):
    """ trust a new PGP key related to a apt-repository """
    if log:
        log_green(
            'trusting keyid %s from %s' % (keyid, keyserver)
        )
    with settings(hide('warnings', 'running', 'stdout')):
        sudo('apt-key adv --keyserver %s --recv %s' % (keyserver, keyid))
    return True


def enable_apt_repositories(prefix, url, version, repositories):
    """ adds an apt repository """
    with settings(hide('warnings', 'running', 'stdout'),
                  warn_only=False, capture=True):
        sudo('apt-add-repository "%s %s %s %s"' % (prefix,
                                                   url,
                                                   version,
                                                   repositories))
        with hide('running', 'stdout'):
            output = sudo("DEBIAN_FRONTEND=noninteractive /usr/bin/apt-get update")
        if 'Some index files failed to download' in output:
            raise SystemExit(1)
        else:
            # if we didn't abort above, we should return True
            return True


def install_gem(gem):
    """ install a particular gem """
    with settings(hide('warnings', 'running', 'stdout', 'stderr'),
                  warn_only=False, capture=True):
        # convert 0 into True, any errors will always raise an exception
        return not bool(
            run("gem install %s --no-rdoc --no-ri" % gem).return_code)


def install_python_module(name):
    """ instals a python module using pip """

    with settings(hide('everything'),
                  warn_only=False, capture=True):
        # convert 0 into True, any errors will always raise an exception
        return not bool(
            run('pip --quiet install %s' % name).return_code)


def install_python_module_locally(name):
    """ instals a python module using pip """
    with settings(hide('everything'),
                  warn_only=False, capture=True):
        # convert 0 into True, any errors will always raise an exception
        print(not bool(local('pip --quiet install %s' % name).return_code))
        return not bool(
            local('pip --quiet install %s' % name).return_code)


def install_system_gem(gem):
    """ install a particular gem """
    with settings(hide('warnings', 'running', 'stdout', 'stderr'),
                  warn_only=False, capture=True):
        # convert 0 into True, any errors will always raise an exception
        return not bool(
            sudo("gem install %s --no-rdoc --no-ri" % gem).return_code)


def install_zfs_from_testing_repository():
    # Enable debugging for ZFS modules
    sudo("echo SPL_DKMS_DISABLE_STRIP=y >> /etc/sysconfig/spl")
    sudo("echo ZFS_DKMS_DISABLE_STRIP=y >> /etc/sysconfig/zfs")
    sudo("yum install --quiet -y --enablerepo=zfs-testing zfs")
    sudo("dkms autoinstall")
    sudo("modprobe zfs")


def is_deb_package_installed(pkg):
    """ checks if a particular deb package is installed """

    with settings(hide('everything'),
                  warn_only=True, capture=True):

        result = sudo('dpkg-query -l "%s" | grep -q ^.i' % pkg)
        return not bool(result.return_code)


def is_package_installed(distribution, pkg):
    """ checks if a particular package is installed """
    if ('centos' in distribution or
            'el' in distribution or
            'redhat' in distribution):
        return(is_rpm_package_installed(pkg))

    if ('ubuntu' in distribution or
            'debian' in distribution):
        return(is_deb_package_installed(pkg))


def is_rpm_package_installed(pkg):
    """ checks if a particular rpm package is installed """

    with settings(hide('warnings', 'running', 'stdout', 'stderr'),
                  warn_only=True, capture=True):

        result = sudo("rpm -q %s" % pkg)
        if result.return_code == 0:
            return True
        elif result.return_code == 1:
            return False
        else:   # print error to user
            print(result)
            raise SystemExit()


def yum_install(**kwargs):
    """
        installs a yum package
    """
    if 'repo' in kwargs:
        repo = kwargs['repo']

    for pkg in list(kwargs['packages']):
        if is_package_installed(distribution='el', pkg=pkg) is False:
            if 'repo' in locals():
                log_green(
                    "installing %s from repo %s ..." % (pkg, repo))
                sudo("yum install -y --quiet --enablerepo=%s %s" % (repo, pkg))
            else:
                log_green("installing %s ..." % pkg)
                sudo("yum install -y --quiet %s" % pkg)


def yum_group_install(**kwargs):
    """ instals a yum group """
    for grp in list(kwargs['groups']):
        log_green("installing %s ..." % grp)
        if 'repo' in kwargs:
            repo = kwargs['repo']
            sudo("yum groupinstall -y --quiet "
                 "--enablerepo=%s '%s'" % (repo, grp))
        else:
            sudo("yum groups mark install -y --quiet '%s'" % grp)
            sudo("yum groups mark convert -y --quiet '%s'" % grp)
            sudo("yum groupinstall -y --quiet '%s'" % grp)


def yum_install_from_url(pkg_name, url):
    """ installs a pkg from a url
        p pkg_name: the name of the package to install
        p url: the full URL for the rpm package
    """
    if is_package_installed(distribution='el', pkg=pkg_name) is False:
        log_green(
            "installing %s from %s" % (pkg_name, url))
        with settings(hide('warnings', 'running', 'stdout', 'stderr'),
                      warn_only=True, capture=True):

            result = sudo("rpm -i %s" % url)
            if result.return_code == 0:
                return True
            elif result.return_code == 1:
                return False
            else:  # print error to user
                print(result)
                raise SystemExit()



from sys import exit
from time import sleep
import uuid

from zope.interface import implementer, provider
from pyrsistent import PClass, field
import pyrax
from novaclient.exceptions import NotFound

from bookshelf.api_v1 import wait_for_ssh
from bookshelf.api_v2.logging_helpers import log_green, log_yellow, log_red
from cloud_instance import ICloudInstance, ICloudInstanceFactory, Distribution


class RackspaceConfiguration(PClass):
    """
    The configuration needed to create a rackspace instance and image.
    """
    username = field(factory=unicode, mandatory=True)
    instance_type = field(factory=unicode, mandatory=True)
    key_pair = field(factory=unicode, mandatory=True)
    public_key_filename = field(factory=unicode, mandatory=True)
    private_key_filename = field(factory=unicode, mandatory=True)
    image_basename = field(type=unicode, mandatory=True, factory=unicode)
    access_key_id = field(factory=unicode, mandatory=True)
    secret_access_key = field(factory=unicode, mandatory=True)
    ami = field(factory=unicode, mandatory=True)
    description = field(factory=unicode, mandatory=True)
    instance_name = field(factory=unicode, mandatory=True)


class RackspaceState(PClass):
    """
    Information about the rackspace instance that will later be used to
    reconnect to the instance.
    """
    instance_name = field(factory=unicode, mandatory=True)
    ip_address = field(factory=unicode, mandatory=True)
    distro = field(factory=unicode, mandatory=True)
    region = field(factory=unicode, mandatory=True)


@implementer(ICloudInstance)
@provider(ICloudInstanceFactory)
class RackspaceInstance(object):

    cloud_type = 'rackspace'

    def __init__(self, config, state):
        self.config = RackspaceConfiguration.create(config)
        self.state = state
        self._nova = self._connect_to_rackspace()

    @property
    def distro(self):
        return Distribution(self.state.distro)

    @property
    def username(self):
        return self.config.username

    @property
    def description(self):
        return self.config.description

    @property
    def ip_address(self):
        return self.state.ip_address

    @property
    def image_basename(self):
        return self.config.image_basename

    @property
    def region(self):
        return self.state.region

    @property
    def key_filename(self):
        return self.config.private_key_filename

    @classmethod
    def create_from_config(cls, config, distro, region):
        distro = distro.value
        instance_name = "{}-{}".format(
            config['instance_name'],
            unicode(uuid.uuid4())
        )
        state = RackspaceState(
            instance_name=instance_name,
            ip_address="",
            distro=distro,
            region=region
        )
        instance = cls(config, state)
        instance.upload_key()
        instance._create_server()
        return instance

    def upload_key(self):
        try:
            log_green("Checking for key pair {}".format(self.config.key_pair))
            self._nova.keypairs.get(self.config.key_pair)
            log_green("Key pair exists in rackspace")
        except NotFound:
            log_green("Creating key pair {}".format(self.config.key_pair))
            with open(self.config.public_key_filename) as keyfile:
                self._nova.keypairs.create(self.config.key_pair,
                                           keyfile.read())

    @classmethod
    def create_from_saved_state(cls, config, saved_state):
        state = RackspaceState.create(saved_state)
        instance = cls(config, state)
        server = instance._nova.servers.find(name=instance.state.instance_name)
        # if we've restarted a terminated server, the ip address
        # might have changed from our saved state, get the
        # networking info and resave the state
        instance._set_instance_networking(server)
        return instance

    def _create_server(self):
        log_yellow("Creating Rackspace instance...")
        flavor = self._nova.flavors.find(name=self.config.instance_type)
        image = self._nova.images.find(name=self.config.ami)
        server = self._nova.servers.create(
            name=self.state.instance_name,
            flavor=flavor.id,
            image=image.id,
            region=self.state.region,
            availability_zone=self.state.region,
            key_name=self.config.key_pair
        )

        while server.status == 'BUILD':
            log_yellow("Waiting for build to finish...")
            sleep(10)
            server = self._nova.servers.get(server.id)
        # check for errors
        if server.status != 'ACTIVE':
            log_red("Error creating rackspace instance")
            exit(1)
        self._set_instance_networking(server)

    def _set_instance_networking(self, server):
        ip_address = server.accessIPv4
        if ip_address is None:
            log_red('No IP address assigned')
            exit(1)
        self.state = self.state.transform(['ip_address'], ip_address)
        wait_for_ssh(ip_address)
        log_green('Connected to server with IP address {0}.'.format(
            ip_address)
        )

    def _connect_to_rackspace(self):
        """ returns a connection object to Rackspace  """
        pyrax.set_setting('identity_type', 'rackspace')
        pyrax.set_default_region(self.state.region)
        pyrax.set_credentials(self.config.access_key_id,
                              self.config.secret_access_key)
        nova = pyrax.connect_to_cloudservers(region=self.state.region)
        return nova

    def create_image(self, image_name):
        server = self._nova.servers.find(name=self.state.instance_name)
        image_id = self._nova.servers.create_image(server.id,
                                                   image_name=image_name)
        image = self._nova.images.get(image_id).status.lower()
        log_green('creating rackspace image...')
        sleep_time = 20
        elapsed = 0
        while self._nova.images.get(image_id).status.lower() not in ['active',
                                                                     'error']:
            log_green('building rackspace image, '
                      'this could take a bit: elapsed {}s.'.format(elapsed))
            sleep(20)
            elapsed += sleep_time
        if image == 'error':
            log_red('error creating image')
            exit(1)

        log_green('finished image: %s' % image_id)
        return image_id

    def list_images(self):
        images = self._nova.images.list()
        log_yellow("creation time\timage_name\timage_id")
        for image in sorted(images, key=lambda x: x.created):
            log_green("{}\t{:50}\t{}".format(
                image.created, image.human_id, image.id)
            )

    def delete_image(self, image_id):
        return self._nova.images.delete(image_id)

    def destroy(self):
        server = self._nova.servers.find(name=self.state.instance_name)
        log_yellow('deleting rackspace instance ...')
        server.delete()

        try:
            while True:
                server = self._nova.servers.get(server.id)
                log_yellow('waiting for deletion ...')
                sleep(5)
        except NotFound:
            pass
        log_green('The server has been deleted')

    def down(self):
        """
        Not raising an exception here because I'm a pushover, settle for
        just yelling loudly at the user.
        """
        log_red("Rackspace instances can't be downed, just keeping it running")
        log_red("/Tableflip :( ")

    def get_state(self):
        """
        Get the state that's needed to reconnect to the instance
        """
        data = {
            'instance_name': self.state.instance_name,
            'ip_address': self.state.ip_address,
            'distro': self.state.distro,
            'region': self.state.region,
        }
        return data



import unittest
from bookshelf.api_v2 import file
from fabric.api import sudo, run
from bookshelf.tests.api_v2.docker_based_tests import (
    with_ephemeral_container,
    prepare_required_docker_images
)


class InsertLineInFileAfterRegexTests(unittest.TestCase):

    @with_ephemeral_container(
        images=['ubuntu-vivid-ruby-ssh',
                'ubuntu-trusty-ruby-ssh',
                'centos-7-ruby-ssh'])
    def test_insert_line_in_file_after_regex_returns_True_when_changed(
            self,
            *args,
            **kwargs):

        contents = '\n'.join(
            [
                'Here I am,',
                'on my bicycle',
                'cool, init?'
            ]
        )

        run('echo "%s" > /tmp/test' % contents)

        # it should return True when changed
        self.assertTrue(
            file.insert_line_in_file_after_regex('/tmp/test',
                                                 'Look mum, no hands',
                                                 '.*on.*bicycle')
        )

    @with_ephemeral_container(
        images=['ubuntu-vivid-ruby-ssh',
                'ubuntu-trusty-ruby-ssh',
                'centos-7-ruby-ssh'])
    def test_insert_line_in_file_after_regex_returns_False_when_not_changed(
            self,
            *args,
            **kwargs):

        contents = '\n'.join(
            [
                'Here I am,',
                'on my bicycle',
                'Look mum, no hands',
                'cool, init?'
            ]
        )

        run('echo "%s" > /tmp/test' % contents)

        # it should return False in nothing changed
        self.assertFalse(
            file.insert_line_in_file_after_regex('/tmp/test',
                                                 'Look mum, no hands',
                                                 '.*on.*bicycle')
        )

    @with_ephemeral_container(
        images=['ubuntu-vivid-ruby-ssh',
                'ubuntu-trusty-ruby-ssh',
                'centos-7-ruby-ssh'])
    def test_insert_line_in_file_after_regex_inserts_line(self,
                                                          *args,
                                                          **kwargs):

        contents = '\n'.join(
            [
                'Here I am,',
                'on my bicycle',
                'cool, init?'
            ]
        )

        expected = '\r\n'.join(
            [
                'Here I am,',
                'on my bicycle',
                'Look mum, no hands',
                'cool, init?'
            ]
        )

        run('echo "%s" > /tmp/test' % contents)

        file.insert_line_in_file_after_regex('/tmp/test',
                                             'Look mum, no hands',
                                             '.*on.*bicycle')

        self.assertEqual(sudo('cat /tmp/test'), expected)

    @with_ephemeral_container(
        images=['ubuntu-vivid-ruby-ssh',
                'ubuntu-trusty-ruby-ssh',
                'centos-7-ruby-ssh'])
    def test_insert_line_in_file_after_regex_raises_Exception_on_failure(
            self,
            *args,
            **kwargs):

        with self.assertRaises(SystemExit) as cm:
            file.insert_line_in_file_after_regex('/tmp/test',
                                                 'Look mum, no hands',
                                                 '.*on.*bicycle')
        self.assertEqual(cm.exception.code, 1)


if __name__ == '__main__':

    prepare_required_docker_images()
    unittest.main(verbosity=4, failfast=True)



import unittest
from bookshelf.api_v2 import git
from fabric.api import run, sudo
from bookshelf.tests.api_v2.docker_based_tests import (
    with_ephemeral_container,
    prepare_required_docker_images
)


class InstallRecentGitFromSourceTests(unittest.TestCase):

    @with_ephemeral_container(
        images=['ubuntu-vivid-ruby-ssh',
                'ubuntu-trusty-ruby-ssh'])
    def test_on_ubuntu_install_recent_git_from_source_installs_git(self,
                                                                   *args,
                                                                   **kwargs):
        sudo('apt-get -y install wget zlib1g-dev libperl-dev')
        sudo('apt-get -y install build-essential')
        git.install_recent_git_from_source(version='2.4.6',
                                           prefix='/usr/local')

        self.assertTrue(
            '2.4.6' in run('git --version')
        )

    @with_ephemeral_container(
        images=['centos-7-ruby-ssh'])
    def test_on_centos_install_recent_git_from_source_installs_git(self,
                                                                   *args,
                                                                   **kwargs):
        sudo('yum -y install wget zlib-devel perl-devel')
        sudo('yum -y groupinstall "Development Tools"')
        git.install_recent_git_from_source(version='2.4.6',
                                           prefix='/usr/local')
        self.assertTrue(
            '2.4.6' in run('git --version')
        )


class GitCloneTests(unittest.TestCase):
    @with_ephemeral_container(
        images=['ubuntu-vivid-ruby-ssh',
                'ubuntu-trusty-ruby-ssh'])
    def test_on_ubuntu_git_clone_clones_repository(self, *args, **kwargs):
        sudo('apt-get -y install git')
        git.git_clone('https://github.com/fabric/fabric.git', 'fabric')

        self.assertTrue(
            'fabric' in run('ls')
        )





if __name__ == '__main__':

    prepare_required_docker_images()
    unittest.main(verbosity=4, failfast=True)




from time import sleep

import boto.ec2
from boto.ec2.blockdevicemapping import BlockDeviceMapping, EBSBlockDeviceType
from pyrsistent import PClass, field, pmap, PMap, pvector, PVector
from zope.interface import implementer, provider

from bookshelf.api_v3.cloud_instance import (
    ICloudInstance, ICloudInstanceFactory, Distribution
)

from bookshelf.api_v2.logging_helpers import log_green, log_yellow, log_red
from bookshelf.api_v2.cloud import wait_for_ssh


class EC2State(PClass):
    """
    Information about the ec2 instance that will later be used to
    reconnect to the instance.
    """
    instance_id = field(type=unicode, mandatory=True, factory=unicode)
    region = field(type=unicode, mandatory=True, factory=unicode)
    distro = field(mandatory=True, factory=Distribution,
                   serializer=lambda _, x: x.value)


class EC2Credentials(PClass):
    """
    The credentials needed to authenticate with EC2.
    """
    access_key_id = field(type=unicode, mandatory=True, factory=unicode)
    secret_access_key = field(type=unicode, mandatory=True, factory=unicode)


def _parse_unicode_pmap(d):
    return pmap({unicode(k): unicode(v)
                 for k, v in d.iteritems()})


def _parse_unicode_pvector(l):
    return pvector([unicode(x) for x in l])


class EC2Configuration(PClass):
    """
    The configuration needed to create an EC2 instance and image.
    """
    credentials = field(type=EC2Credentials, mandatory=True)
    username = field(type=unicode, mandatory=True, factory=unicode)
    instance_name = field(type=unicode, mandatory=True, factory=unicode)
    tags = field(type=PMap, mandatory=True, factory=_parse_unicode_pmap)
    image_description = field(type=unicode, mandatory=True, factory=unicode)
    image_basename = field(type=unicode, mandatory=True, factory=unicode)
    ami = field(type=unicode, mandatory=True, factory=unicode)
    key_filename = field(type=unicode, mandatory=True, factory=unicode)
    key_pair = field(type=unicode, mandatory=True, factory=unicode)
    instance_type = field(type=unicode, mandatory=True, factory=unicode)
    disk_name = field(type=unicode, mandatory=True, factory=unicode)
    disk_size = field(type=int, mandatory=True, factory=int)
    security_groups = field(type=PVector, mandatory=True,
                            factory=_parse_unicode_pvector)


def _connect_to_ec2(region, credentials):
    """
    :param region: The region of AWS to connect to.
    :param EC2Credentials credentials: The credentials to use to authenticate
        with EC2.

    :return: a connection object to AWS EC2
    """
    conn = boto.ec2.connect_to_region(
        region,
        aws_access_key_id=credentials.access_key_id,
        aws_secret_access_key=credentials.secret_access_key
    )
    if conn:
        return conn
    else:
        log_red('Failure to authenticate to EC2.')
        return False


def _create_server_ec2(connection,
                       region,
                       disk_name,
                       disk_size,
                       ami,
                       key_pair,
                       instance_type,
                       tags={},
                       security_groups=None,
                       delete_on_termination=True,
                       log=False,
                       wait_for_ssh_available=True):
    """
    Creates EC2 Instance
    """

    if log:
        log_green("Started...")
        log_yellow("...Creating EC2 instance...")

    ebs_volume = EBSBlockDeviceType()
    ebs_volume.size = disk_size
    bdm = BlockDeviceMapping()
    bdm[disk_name] = ebs_volume

    # get an ec2 ami image object with our choosen ami
    image = connection.get_all_images(ami)[0]
    # start a new instance
    reservation = image.run(1, 1,
                            key_name=key_pair,
                            security_groups=security_groups,
                            block_device_map=bdm,
                            instance_type=instance_type)

    # and get our instance_id
    instance = reservation.instances[0]

    #  and loop and wait until ssh is available
    while instance.state == u'pending':
        if log:
            log_yellow("Instance state: %s" % instance.state)
        sleep(10)
        instance.update()
    if log:
        log_green("Instance state: %s" % instance.state)
    if wait_for_ssh_available:
        wait_for_ssh(instance.public_dns_name)

    # update the EBS volumes to be deleted on instance termination
    if delete_on_termination:
        for dev, bd in instance.block_device_mapping.items():
            instance.modify_attribute('BlockDeviceMapping',
                                      ["%s=%d" % (dev, 1)])

    # add a tag to our instance
    if tags:
        connection.create_tags([instance.id], tags)

    if log:
        log_green("Public dns: %s" % instance.public_dns_name)

    # returns our new instance
    return instance


@implementer(ICloudInstance)
@provider(ICloudInstanceFactory)
class EC2Instance():
    """
    Class representing an EC2 instance, that provides methods for interacting
    with the instance.

    :ivar connection: A boto connection object to interact with ec2.
    :ivar instance: A boto instance object.
    :ivar config: An EC2Configuration describing the configuration for this EC2
        instance.
    :ivar state: An EC2State describing this EC2 instance.
    """
    def __init__(self, instance, connection, config, state):
        self.connection = connection
        self.instance = instance
        self.config = config
        self.state = state

    cloud_type = u'ec2'

    @property
    def ip_address(self):
        return self.instance.ip_address

    @property
    def image_basename(self):
        return self.config.image_basename

    @property
    def username(self):
        return self.config.username

    @property
    def key_filename(self):
        return self.config.key_filename

    @property
    def distro(self):
        return self.state.distro

    @property
    def region(self):
        return self.state.region

    @classmethod
    def create_from_config(cls, config, distro, region):
        parsed_config = EC2Configuration.create(config)
        connection = _connect_to_ec2(
            region=region,
            credentials=parsed_config.credentials
        )
        instance = _create_server_ec2(
            connection=connection,
            region=region,
            disk_name=parsed_config.disk_name,
            disk_size=parsed_config.disk_size,
            ami=parsed_config.ami,
            key_pair=parsed_config.key_pair,
            instance_type=parsed_config.instance_type,
            tags=parsed_config.tags,
            security_groups=parsed_config.security_groups,
            delete_on_termination=True,
            log=False,
            wait_for_ssh_available=True
        )
        state = EC2State(
            instance_id=instance.id,
            region=region,
            distro=distro,
        )

        return cls(
            connection=connection,
            instance=instance,
            config=parsed_config,
            state=state
        )

    @classmethod
    def create_from_saved_state(cls, config, saved_state, timeout=600):
        parsed_config = EC2Configuration.create(config)
        state = EC2State.create(saved_state)
        connection = _connect_to_ec2(
            region=state.region,
            credentials=parsed_config.credentials
        )

        instance = connection.start_instances(
            instance_ids=state.instance_id)[0]
        instance.update()
        while instance.state != "running" and timeout > 1:
            log_yellow("Instance state: %s" % instance.state)
            sleep(10)
            timeout = timeout - 10
            instance.update()

        # and make sure we don't return until the instance is fully up
        wait_for_ssh(instance.ip_address)
        return cls(
            connection=connection,
            instance=instance,
            config=parsed_config,
            state=state
        )

    def create_image(self, image_name):
        ami = self.connection.create_image(
            self.state.instance_id,
            image_name,
            description=self.config.image_description,
        )

        image_status = self.connection.get_image(ami)
        while (image_status.state != "available" and
               image_status.state != "failed"):
            log_yellow('creating ami...')
            sleep(60)
            image_status = self.connection.get_image(ami)

        if image_status.state == "available":
            log_green("ami %s %s" % (ami, image_status))
            return(ami)
        else:
            log_red("ami %s %s" % (ami, image_status))
            return False

    def list_images(self):
        images = self.connection.get_all_images(owners='self')
        log_yellow("creation time\timage_name\timage_id")
        for image in sorted(images, key=lambda x: x.creationDate):
            log_green("{}\t{:50}\t{}".format(
                image.creationDate, image.name, image.id)
            )

    def delete_image(self, image_id):
        images = self.connection.get_all_images(owners='self')
        found = False
        for image in images:
            if image.id == image_id:
                log_yellow("Deleting image {}".format(image_id))
                image.deregister(delete_snapshot=True)
                found = True
                break
        if not found:
            log_red("Could not find image {}".format(image_id))

    def _ebs_volume_exists(self, volume_id):
        """ finds out if a ebs volume exists """
        for vol in self.connection.get_all_volumes():
            if vol.id == volume_id:
                return True
        return False

    def _destroy_ebs_volume(self, volume_id):
        """ destroys an ebs volume """
        if self._ebs_volume_exists(volume_id):
            log_yellow('destroying EBS volume ...')
            try:
                self.connection.delete_volume(volume_id)
            except Exception as e:
                # our EBS volume may be gone, but AWS info tables are stale
                # wait a bit and ask again
                log_yellow("exception raised when deleting volume")
                log_yellow("{} -- {}".format(type(e), str(e)))
                worked = False
                for i in range(6):
                    sleep(5)
                    if not self._ebs_volume_exists(volume_id):
                        log_green("It worked that time")
                        worked = True
                if not worked:
                    raise Exception("Couldn't delete EBS volume")

    def destroy(self):
        self.down()
        volumes = self.connection.get_all_volumes(
            filters={'attachment.instance-id': self.state.instance_id}
        )
        instance = self.connection.terminate_instances(
            instance_ids=[self.state.instance_id])[0]
        log_yellow('destroying instance ...')
        while instance.state != "terminated":
            log_yellow("Instance state: %s" % instance.state)
            sleep(10)
            instance.update()
        for volume in volumes:
            self._destroy_ebs_volume(volume.id)

    def down(self):
        instance = self.connection.stop_instances(
            instance_ids=self.state.instance_id)[0]
        while instance.state != "stopped":
            log_yellow("Instance state: %s" % instance.state)
            sleep(10)
            instance.update()
        log_green('Instance state: %s' % instance.state)

    def get_state(self):
        return self.state.serialize()



# vim: ai ts=4 sts=4 et sw=4 ft=python fdm=indent et foldlevel=0

from fabric.api import sudo, settings, run
from fabric.context_managers import hide
from fabric.contrib.files import (
                                  contains)
from bookshelf.api_v2.logging_helpers import (log_green,
                                              log_red)
from bookshelf.api_v2.os_helpers import systemd


def cache_docker_image_locally(docker_image, log=False):
    if log:
        log_green('pulling docker image %s locally' % docker_image)
    sudo("docker pull %s" % docker_image)


def create_docker_group():
    """ creates the docker group """
    if not contains('/etc/group', 'docker', use_sudo=True):
        sudo("groupadd docker")


def does_container_exist(container):
    with settings(warn_only=True):
        result = sudo('docker inspect %s' % container)
        print('*********************************************')
        log_red(result.return_code)
    if result.return_code is 0:
        return True
    else:
        return False


def does_image_exist(image):
    with settings(warn_only=True):
        if image in sudo('docker images'):
            return True
        else:
            return False


def get_container_id(container):
        with hide('running', 'stdout'):
            result = sudo(
                "docker ps -a | grep %s | awk '{print $1}'" % container)
            return result


def get_image_id(image):
        result = sudo("docker images | grep %s | awk '{print $3}'" % image)
        return result


def install_docker():
    """ installs docker """
    with settings(hide('running', 'stdout')):
        run('curl https://get.docker.com/ > /tmp/install-docker.sh')
        sudo('sh /tmp/install-docker.sh')
        systemd('docker.service')


def remove_image(image):
    sudo('docker rmi -f %s' % get_image_id(image))


def remove_container(container):
    sudo('docker rm -f %s' % get_container_id(container))




import unittest

from zope.interface.verify import verifyObject, verifyClass

from bookshelf.api_v3.gce import GCEInstance
from bookshelf.api_v3.ec2 import EC2Instance
from bookshelf.api_v3.rackspace import RackspaceInstance
from bookshelf.api_v3.cloud_instance import (
    ICloudInstanceFactory,
    ICloudInstance
)


class TestGCEInterfaces(unittest.TestCase):

    def test_gce_provides_cloud_instance_factory(self):
        verifyObject(ICloudInstanceFactory, GCEInstance)

    def test_gce_implements_cloud_instance(self):
        verifyClass(ICloudInstance, GCEInstance)


class TestEC2Interfaces(unittest.TestCase):

    def test_ec2_provides_cloud_instance_factory(self):
        verifyObject(ICloudInstanceFactory, EC2Instance)

    def test_ec2_implements_cloud_instance(self):
        verifyClass(ICloudInstance, EC2Instance)


class TestRackspaceInterfaces(unittest.TestCase):

    def test_rackspace_provides_cloud_instance_factory(self):
        verifyObject(ICloudInstanceFactory, RackspaceInstance)

    def test_rackspaceee_implements_cloud_instance(self):
        verifyClass(ICloudInstance, RackspaceInstance)


if __name__ == '__main__':
    unittest.main(verbosity=4, failfast=True)




import unittest
from bookshelf.api_v2 import python
from fabric.api import sudo, run
from bookshelf.tests.api_v2.docker_based_tests import (
    with_ephemeral_container,
    prepare_required_docker_images
)


class UpdateSystemPipToLatestPipUbuntuTests(unittest.TestCase):
    @with_ephemeral_container(
        images=['ubuntu-vivid-ruby-ssh',
                'ubuntu-trusty-ruby-ssh'])
    def test_update_system_pip_to_latest_pip(self, *args, **kwargs):
        python.update_system_pip_to_latest_pip()

        self.assertRegexpMatches(
            sudo('pip --version'),
            'pip 8.* from.*'
        )


class UpdateToLatestPipUbuntuTests(unittest.TestCase):
    @with_ephemeral_container(
        images=['ubuntu-vivid-ruby-ssh',
                'ubuntu-trusty-ruby-ssh'])
    def test_update_to_latest_pip(self, *args, **kwargs):
        python.update_to_latest_pip()

        self.assertRegexpMatches(
            run('pip --version'),
            'pip 8.* from.*'
        )


if __name__ == '__main__':

    prepare_required_docker_images()
    unittest.main(verbosity=4, failfast=True)



import unittest
from fabric.api import sudo
from bookshelf.api_v2.docker_based_tests import DockerBasedTests
from bookshelf.api_v2.decorators import with_ephemeral_container
from bookshelf.api_v2 import os_helpers


class AddUsrLocalBinToPathTests(DockerBasedTests):

    @with_ephemeral_container(
        distributions=['ubuntu-vivid', 'ubuntu-trusty', 'centos-7'])
    def test_add_usr_local_bin_to_path_updates_PATH(self, *args, **kwargs):
        os_helpers.add_usr_local_bin_to_path()
        self.assertTrue('/usr/local/bin' in sudo('echo $PATH'))

    @with_ephemeral_container(
        distributions=['ubuntu-vivid', 'ubuntu-trusty', 'centos-7'])
    def test_add_user_local_bin_returns_True_on_success(self, *args, **kwargs):
        self.assertTrue(os_helpers.add_usr_local_bin_to_path())

    @with_ephemeral_container(
        distributions=['ubuntu-vivid', 'ubuntu-trusty', 'centos-7'])
    def test_add_usr_local_bin_raises_exception_on_failure(self, *args,
                                                           **kwargs):
        # force an exception by removing /etc/profile.d
        sudo('rm -rf /etc/profile.d')

        with self.assertRaises(SystemExit) as cm:
            os_helpers.add_usr_local_bin_to_path()
        self.assertEqual(cm.exception.code, 1)


class ArchTests(DockerBasedTests):

    @with_ephemeral_container(
        distributions=['ubuntu-vivid', 'ubuntu-trusty', 'centos-7'])
    def test_arch(self, *args, **kwargs):
        self.assertEquals(os_helpers.arch(), 'x86_64')


class DirAttribsTests(DockerBasedTests):

    @with_ephemeral_container(
        distributions=['ubuntu-vivid', 'ubuntu-trusty', 'centos-7'])
    def test_dir_attribs_with_sudo_updates_perms(self, *args, **kwargs):
        # make sure we have a user1 and group1 on our docker instance
        sudo('/usr/sbin/useradd user1')
        sudo('/usr/sbin/groupadd group1')
        sudo('echo > /tmp/temp1')

        os_helpers.dir_attribs(location='/tmp/temp1',
                                        mode='755',
                                        owner='user1',
                                        group='group1',
                                        recursive=False,
                                        use_sudo=True)

        perms, x, owner, group = sudo('ls -ld /tmp/temp1').split(' ')[0:4]
        self.assertEquals(perms, '-rwxr-xr-x')
        self.assertEquals(owner, 'user1')
        self.assertEquals(group, 'group1')

    @with_ephemeral_container(
        distributions=['ubuntu-vivid', 'ubuntu-trusty', 'centos-7'])
    def test_dir_attribs_with_sudo_updates_perms_recursively(self,
                                                             *args,
                                                             **kwargs):
        # make sure we have a user1 and group1 on our docker instance
        sudo('/usr/sbin/useradd user1')
        sudo('/usr/sbin/groupadd group1')
        sudo('mkdir /tmp/temp1')
        sudo('echo > /tmp/temp1/file1')

        os_helpers.dir_attribs(location='/tmp/temp1',
                                        mode='755',
                                        owner='user1',
                                        group='group1',
                                        recursive=True,
                                        use_sudo=True)

        perms, x, owner, group = sudo('ls -ld /tmp/temp1').split(' ')[0:4]
        self.assertEquals(perms, 'drwxr-xr-x')
        self.assertEquals(owner, 'user1')
        self.assertEquals(group, 'group1')

        perms, x, owner, group = sudo('ls -l /tmp/temp1/file1').split(' ')[0:4]
        self.assertEquals(perms, '-rwxr-xr-x')
        self.assertEquals(owner, 'user1')
        self.assertEquals(group, 'group1')

    @with_ephemeral_container(
        distributions=['ubuntu-vivid', 'ubuntu-trusty', 'centos-7'])
    def test_dir_attribs_without_sudo_updates_perms(self, *args, **kwargs):
        # make sure we have a user1 and group1 on our docker instance
        sudo('/usr/sbin/useradd user1')
        sudo('/usr/sbin/groupadd group1')
        sudo('echo > /tmp/temp1')

        os_helpers.dir_attribs(location='/tmp/temp1',
                                        mode='755',
                                        owner='user1',
                                        group='group1',
                                        recursive=False,
                                        use_sudo=False)

        perms, x, owner, group = sudo('ls -l /tmp/temp1').split(' ')[0:4]
        self.assertEquals(perms, '-rwxr-xr-x')
        self.assertEquals(owner, 'user1')
        self.assertEquals(group, 'group1')

    @with_ephemeral_container(
        distributions=['ubuntu-vivid', 'ubuntu-trusty', 'centos-7'])
    def test_dir_attribs_without_sudo_updates_perms_recursively(self,
                                                                *args,
                                                                **kwargs):
        # make sure we have a user1 and group1 on our docker instance
        sudo('/usr/sbin/useradd user1')
        sudo('/usr/sbin/groupadd group1')
        sudo('mkdir /tmp/temp1')
        sudo('echo > /tmp/temp1/file1')

        os_helpers.dir_attribs(location='/tmp/temp1',
                                        mode='755',
                                        owner='user1',
                                        group='group1',
                                        recursive=True,
                                        use_sudo=False)
        perms, x, owner, group = sudo('ls -ld /tmp/temp1').split(' ')[0:4]
        self.assertEquals(perms, 'drwxr-xr-x')
        self.assertEquals(owner, 'user1')
        self.assertEquals(group, 'group1')

        perms, x, owner, group = sudo('ls -l /tmp/temp1/file1').split(' ')[0:4]
        self.assertEquals(perms, '-rwxr-xr-x')
        self.assertEquals(owner, 'user1')
        self.assertEquals(group, 'group1')

    @with_ephemeral_container(
        distributions=['ubuntu-vivid', 'ubuntu-trusty', 'centos-7'])
    def test_dir_attribs_returns_True_on_sucess(self, *args, **kwargs):
        # make sure we have a user1 and group1 on our docker instance
        sudo('/usr/sbin/useradd user1')
        sudo('/usr/sbin/groupadd group1')
        sudo('mkdir /tmp/temp1')
        sudo('echo > /tmp/temp1/file1')

        result = os_helpers.dir_attribs(location='/tmp/temp1',
                                        mode='755',
                                        owner='user1',
                                        group='group1',
                                        recursive=True,
                                        use_sudo=True)

        self.assertTrue(result)

    @with_ephemeral_container(
        distributions=['ubuntu-vivid', 'ubuntu-trusty', 'centos-7'])
    def test_dir_attribs_raises_exception_on_failure(self, *args,
                                                     **kwargs):
        with self.assertRaises(SystemExit) as cm:
            os_helpers.dir_attribs(location='/tmp/no-valid-path',
                                            mode='755',
                                            owner='not-a-valid-user',
                                            group='not-a-valid-group',
                                            recursive=True,
                                            use_sudo=False)
        self.assertEqual(cm.exception.code, 1)


class DirEnsureTests(DockerBasedTests):

    @with_ephemeral_container(
        distributions=['ubuntu-vivid', 'ubuntu-trusty', 'centos-7'])
    def test_dir_ensure_creates_dir_if_missing(self, *args, **kwargs):
        # make sure we have a user1 and group1 on our docker instance
        sudo('/usr/sbin/useradd user1')
        sudo('/usr/sbin/groupadd group1')
        os_helpers.dir_ensure(location='/tmp/temp1',
                                       mode='755',
                                       owner='user1',
                                       group='group1',
                                       recursive=False,
                                       use_sudo=False)
        self.assertTrue(sudo('ls -d /tmp/temp1'))

    @with_ephemeral_container(
        distributions=['ubuntu-vivid', 'ubuntu-trusty', 'centos-7'])
    def test_dir_ensure_creates_dir_if_missing_as_root(self, *args, **kwargs):
        # make sure we have a user1 and group1 on our docker instance
        sudo('/usr/sbin/useradd user1')
        sudo('/usr/sbin/groupadd group1')
        os_helpers.dir_ensure(location='/tmp/temp1',
                                       mode='755',
                                       owner='user1',
                                       group='group1',
                                       recursive=False,
                                       use_sudo=True)
        self.assertTrue(sudo('ls -d /tmp/temp1'))

    @with_ephemeral_container(
        distributions=['ubuntu-vivid', 'ubuntu-trusty', 'centos-7'])
    def test_dir_ensure_creates_dir_recursive_if_missing(self, *args, **kwargs):
        # make sure we have a user1 and group1 on our docker instance
        sudo('/usr/sbin/useradd user1')
        sudo('/usr/sbin/groupadd group1')
        os_helpers.dir_ensure(location='/tmp/level1/level2',
                                       mode='755',
                                       owner='user1',
                                       group='group1',
                                       recursive=True,
                                       use_sudo=False)
        self.assertTrue(sudo('ls -d /tmp/level1/level2'))

    @with_ephemeral_container(
        distributions=['ubuntu-vivid', 'ubuntu-trusty', 'centos-7'])
    def test_dir_ensure_creates_dir_recursive_if_missing_as_root(self,
                                                                 *args,
                                                                 **kwargs):
        # make sure we have a user1 and group1 on our docker instance
        sudo('/usr/sbin/useradd user1')
        sudo('/usr/sbin/groupadd group1')
        os_helpers.dir_ensure(location='/tmp/level1/level2',
                                       mode='755',
                                       owner='user1',
                                       group='group1',
                                       recursive=True,
                                       use_sudo=True)
        self.assertTrue(sudo('ls -d /tmp/level1/level2'))

    @with_ephemeral_container(
        distributions=['ubuntu-vivid', 'ubuntu-trusty', 'centos-7'])
    def test_dir_ensure_with_sudo_updates_perms(self, *args, **kwargs):
        # make sure we have a user1 and group1 on our docker instance
        sudo('/usr/sbin/useradd user1')
        sudo('/usr/sbin/groupadd group1')

        os_helpers.dir_ensure(location='/tmp/temp1',
                                       mode='700',
                                       owner='user1',
                                       group='group1',
                                       recursive=False,
                                       use_sudo=True)

        perms, x, owner, group = sudo('ls -ld /tmp/temp1').split(' ')[0:4]
        self.assertEquals(perms, 'drwx------')
        self.assertEquals(owner, 'user1')
        self.assertEquals(group, 'group1')

    @with_ephemeral_container(
        distributions=['ubuntu-vivid', 'ubuntu-trusty', 'centos-7'])
    def test_dir_ensure_with_sudo_updates_perms_recursively(self,
                                                            *args,
                                                            **kwargs):
        # make sure we have a user1 and group1 on our docker instance
        sudo('/usr/sbin/useradd user1')
        sudo('/usr/sbin/groupadd group1')
        sudo('mkdir /tmp/level1')
        sudo('mkdir /tmp/level1/level2')

        os_helpers.dir_ensure(location='/tmp/level1',
                                       mode='700',
                                       owner='user1',
                                       group='group1',
                                       recursive=True,
                                       use_sudo=True)

        for folder in ['/tmp/level1', '/tmp/level1/level2']:
            perms, x, owner, group = sudo('ls -ld %s' % folder).split(' ')[0:4]
            self.assertEquals(perms, 'drwx------')
            self.assertEquals(owner, 'user1')
            self.assertEquals(group, 'group1')

    @with_ephemeral_container(
        distributions=['ubuntu-vivid', 'ubuntu-trusty', 'centos-7'])
    def test_dir_ensure_without_sudo_updates_perms(self, *args, **kwargs):
        # make sure we have a user1 and group1 on our docker instance
        sudo('/usr/sbin/useradd user1')
        sudo('/usr/sbin/groupadd group1')

        os_helpers.dir_ensure(location='/tmp/temp1',
                                       mode='700',
                                       owner='user1',
                                       group='group1',
                                       recursive=False,
                                       use_sudo=False)

        perms, x, owner, group = sudo('ls -ld /tmp/temp1').split(' ')[0:4]
        self.assertEquals(perms, 'drwx------')
        self.assertEquals(owner, 'user1')
        self.assertEquals(group, 'group1')

    @with_ephemeral_container(
        distributions=['ubuntu-vivid', 'ubuntu-trusty', 'centos-7'])
    def test_dir_ensure_without_sudo_updates_perms_recursively(self,
                                                               *args,
                                                               **kwargs):
        # make sure we have a user1 and group1 on our docker instance
        sudo('/usr/sbin/useradd user1')
        sudo('/usr/sbin/groupadd group1')
        sudo('mkdir -p /tmp/level1/level2')

        os_helpers.dir_ensure(location='/tmp/level1',
                                       mode='700',
                                       owner='user1',
                                       group='group1',
                                       recursive=True,
                                       use_sudo=False)
        for folder in ['/tmp/level1', '/tmp/level1/level2']:
            perms, x, owner, group = sudo('ls -ld %s' % folder).split(' ')[0:4]
            self.assertEquals(perms, 'drwx------')
            self.assertEquals(owner, 'user1')
            self.assertEquals(group, 'group1')

    @with_ephemeral_container(
        distributions=['ubuntu-vivid', 'ubuntu-trusty', 'centos-7'])
    def test_dir_ensure_returns_True_on_sucess(self, *args, **kwargs):
        # make sure we have a user1 and group1 on our docker instance
        sudo('/usr/sbin/useradd user1')
        sudo('/usr/sbin/groupadd group1')
        sudo('mkdir /tmp/temp1')

        result = os_helpers.dir_ensure(location='/tmp/temp1',
                                       mode='755',
                                       owner='user1',
                                       group='group1',
                                       recursive=True,
                                       use_sudo=True)
        self.assertTrue(result)

    @with_ephemeral_container(
        distributions=['ubuntu-vivid', 'ubuntu-trusty', 'centos-7'])
    def test_dir_ensure_raises_exception_on_failure(self, *args, **kwargs):
        with self.assertRaises(SystemExit) as cm:
            os_helpers.dir_ensure(location='/tmp/no-valid-path',
                                           mode='755',
                                           owner='not-a-valid-user',
                                           group='not-a-valid-group',
                                           recursive=True,
                                           use_sudo=False)
        self.assertEqual(cm.exception.code, 1)


class DirExistsTests(DockerBasedTests):

    @with_ephemeral_container(
        distributions=['ubuntu-vivid', 'ubuntu-trusty', 'centos-7'])
    def test_dir_exists_returns_True(self, *args, **kwargs):
        # make sure we have a /tmp/temp1 folder
        sudo('mkdir /tmp/temp1')
        self.assertTrue(
            os_helpers.dir_exists(location='/tmp/temp1',
                                  use_sudo=False)
        )

    @with_ephemeral_container(
        distributions=['ubuntu-vivid', 'ubuntu-trusty', 'centos-7'])
    def test_dir_exists_returns_False_if_missing(self, *args, **kwargs):
        self.assertFalse(
            os_helpers.dir_exists(location='/tmp/temp1',
                                  use_sudo=False)
        )

    @with_ephemeral_container(
        distributions=['ubuntu-vivid', 'ubuntu-trusty', 'centos-7'])
    def test_dir_exists_returns_True_with_sudo(self, *args, **kwargs):
        # make sure we have a level2 folder only accessible by root
        sudo('mkdir -p /tmp/level1/level2')
        sudo('chmod 700 /tmp/level1')
        self.assertTrue(
            os_helpers.dir_exists(location='/tmp/level1/level2',
                                  use_sudo=True)
        )

    @with_ephemeral_container(
        distributions=['ubuntu-vivid', 'ubuntu-trusty', 'centos-7'])
    def test_dir_exists_returns_False_if_missing_with_sudo(self,
                                                           *args,
                                                           **kwargs):
        self.assertFalse(
            os_helpers.dir_exists(location='/tmp/level1/level2',
                                  use_sudo=False)
        )

    @with_ephemeral_container(
        distributions=['ubuntu-vivid', 'ubuntu-trusty', 'centos-7'])
    def test_dir_exists_returns_False_without_sudo_when_folder_is_root_owned(
            self, *args, **kwargs):
        # make sure we have a level2 folder only accessible by root
        sudo('mkdir -p /tmp/level1/level2')
        sudo('chmod 700 /tmp/level1')
        self.assertTrue(
            os_helpers.dir_exists(location='/tmp/level1/level2',
                                  use_sudo=False)
        )


class DisableEnvResetOnSudoTests(DockerBasedTests):

    @with_ephemeral_container(
        distributions=['ubuntu-vivid', 'ubuntu-trusty', 'centos-7'])
    def test_disable_env_reset_on_sudo_updates_sudoers(self, *args, **kwargs):
        # distros have different /etc/sudoers file contents, so we will make
        # sure we target the correct contents for each linux distribution.
        os = sudo('cat /etc/os-release').lower()

        # update /etc/sudoers
        os_helpers.disable_env_reset_on_sudo()

        # we rely on md5sum of the file so check if our changes were applied
        # correctly
        md5sum = sudo('md5sum /etc/sudoers').split(' ')[0]
        if 'ubuntu' in os:
            self.assertEquals(md5sum,
                              '5328c9fb99ad099d2d18a45bc67ee024')
        if 'centos' in os:
            sudo('md5sum /etc/sudoers')
            self.assertEquals(md5sum,
                              '8d7a8d9630ccb427c5adbeccf6846415')

    @with_ephemeral_container(
        distributions=['ubuntu-vivid', 'ubuntu-trusty', 'centos-7'])
    def test_disable_env_reset_on_sudo_is_idempodent(self, *args, **kwargs):
        # distros have different /etc/sudoers file contents, so we will make
        # sure we target the correct contents for each linux distribution.
        os = sudo('cat /etc/os-release').lower()

        # update /etc/sudoers twice.
        # the entries on /etc/sudoers should only be inserted once
        os_helpers.disable_env_reset_on_sudo()

        # we rely on md5sum of the file so check if our changes were applied
        # correctly
        md5sum = sudo('md5sum /etc/sudoers').split(' ')[0]
        if 'ubuntu' in os:
            self.assertEquals(md5sum,
                              '5328c9fb99ad099d2d18a45bc67ee024')
        if 'centos' in os:
            sudo('md5sum /etc/sudoers')
            self.assertEquals(md5sum,
                              '8d7a8d9630ccb427c5adbeccf6846415')

    @with_ephemeral_container(
        distributions=['ubuntu-vivid', 'ubuntu-trusty', 'centos-7'])
    def test_disable_env_reset_returns_True(self, *args, **kwargs):
        self.assertTrue(os_helpers.disable_env_reset_on_sudo())

    @with_ephemeral_container(
        distributions=['ubuntu-vivid', 'ubuntu-trusty', 'centos-7'])
    def test_disable_env_reset_on_sudo_raises_exception_on_failure(self,
                                                                   *args,
                                                                   **kwargs):
        # this will make if fail
        sudo('rm /etc/sudoers')
        with self.assertRaises(SystemExit) as cm:
            os_helpers.disable_env_reset_on_sudo()
        self.assertEqual(cm.exception.code, 1)


class DisableRequirettyOnSudoersTests(DockerBasedTests):

    @with_ephemeral_container(
        distributions=['ubuntu-vivid', 'ubuntu-trusty', 'centos-7'])
    def test_disable_requiretty_on_sudoers_updates_sudoers(self,
                                                           *args,
                                                           **kwargs):
        # distros have different /etc/sudoers file contents, so we will make
        # sure we target the correct contents for each linux distribution.
        os = sudo('cat /etc/os-release').lower()

        sudo('cat /etc/sudoers')

        # we rely on md5sum of the file so check if our changes were applied
        # correctly
        md5sum = sudo('md5sum /etc/sudoers').split(' ')[0]
        if 'ubuntu' in os:
            # our docker image doesn't contain the entry we need so...
            sudo('echo "Defaults    requiretty" >> /etc/sudoers')

            # update /etc/sudoers
            os_helpers.disable_requiretty_on_sudoers()

            self.assertEquals(md5sum,
                              'e8e73f16ed73309df7574c12fbcc0af7')
        if 'centos' in os:
            sudo('md5sum /etc/sudoers')

            # update /etc/sudoers
            os_helpers.disable_requiretty_on_sudoers()

            sudo('cat /etc/sudoers')
            self.assertEquals(md5sum,
                              'ef817e657e3ffa6b0a88f59e3fc7241b')

    @with_ephemeral_container(
        distributions=['ubuntu-vivid', 'ubuntu-trusty', 'centos-7'])
    def test_disable_requiretty_on_sudoers_is_idempodent(self,
                                                         *args,
                                                         **kwargs):
        # distros have different /etc/sudoers file contents, so we will make
        # sure we target the correct contents for each linux distribution.
        os = sudo('cat /etc/os-release').lower()

        # we rely on md5sum of the file so check if our changes were applied
        # correctly
        md5sum = sudo('md5sum /etc/sudoers').split(' ')[0]
        if 'ubuntu' in os:
            # our docker image doesn't contain the entry we need so...
            sudo('echo "Defaults    requiretty" >> /etc/sudoers')

            # update /etc/sudoers, twice
            # the entries on /etc/sudoers should only be inserted once
            os_helpers.disable_requiretty_on_sudoers()
            os_helpers.disable_requiretty_on_sudoers()

            self.assertEquals(md5sum,
                              'e8e73f16ed73309df7574c12fbcc0af7')
        if 'centos' in os:
            # update /etc/sudoers
            # the entries on /etc/sudoers should only be inserted once
            os_helpers.disable_requiretty_on_sudoers()
            os_helpers.disable_requiretty_on_sudoers()

            self.assertEquals(md5sum,
                              'ef817e657e3ffa6b0a88f59e3fc7241b')

    @with_ephemeral_container(
        distributions=['ubuntu-vivid', 'ubuntu-trusty', 'centos-7'])
    def test_disable_requiretty_on_sudoers_returns_True(self, *args, **kwargs):
        self.assertTrue(os_helpers.disable_requiretty_on_sudoers())

    @with_ephemeral_container(
        distributions=['ubuntu-vivid', 'ubuntu-trusty', 'centos-7'])
    def test_disable_requiretty_on_sudoers_raises_exception_on_failure(
            self, *args, **kwargs):
        # this will make if fail
        sudo('rm /etc/sudoers')
        with self.assertRaises(SystemExit) as cm:
            os_helpers.disable_requiretty_on_sudoers()
        self.assertEqual(cm.exception.code, 1)


class DisableRequirettyOnSshdConfigTests(DockerBasedTests):
    @with_ephemeral_container(
        distributions=['ubuntu-vivid', 'ubuntu-trusty', 'centos-7'])
    def test_disable_requiretty_on_sshd_config_updates_sshd_config(self,
                                                                   *args,
                                                                   **kwargs):
        # distros have different /etc/sshd_config file contents, so we will make
        # sure we target the correct contents for each linux distribution.
        os = sudo('cat /etc/os-release').lower()

        # this is missing on our docker images
        sudo('echo "Requiretty yes" >> /etc/ssh/sshd_config')
        # update /etc/sshd_config
        os_helpers.disable_requiretty_on_sshd_config()
        # we rely on md5sum of the file so check if our changes were applied
        # correctly
        md5sum = sudo('md5sum /etc/ssh/sshd_config').split(' ')[0]

        if 'ubuntu' in os:
            self.assertEquals(md5sum,
                              '208abf29a29805dbfe4ac954bb50b0c9')
        if 'centos' in os:
            self.assertEquals(md5sum,
                              '9af1ec6efaf3bf271fc1838613fa0778')

    @with_ephemeral_container(
        distributions=['ubuntu-vivid', 'ubuntu-trusty', 'centos-7'])
    def test_disable_requiretty_on_sshd_config_is_idempodent(self,
                                                             *args,
                                                             **kwargs):
        # distros have different /etc/sshd_config file contents, so we will make
        # sure we target the correct contents for each linux distribution.
        os = sudo('cat /etc/os-release').lower()

        # this is missing on our docker images
        sudo('echo "Requiretty yes" >> /etc/ssh/sshd_config')
        # update /etc/sshd_config, twice
        os_helpers.disable_requiretty_on_sshd_config()
        os_helpers.disable_requiretty_on_sshd_config()
        # we rely on md5sum of the file so check if our changes were applied
        # correctly
        md5sum = sudo('md5sum /etc/ssh/sshd_config').split(' ')[0]

        if 'ubuntu' in os:
            self.assertEquals(md5sum,
                              '208abf29a29805dbfe4ac954bb50b0c9')
        if 'centos' in os:
            self.assertEquals(md5sum,
                              '9af1ec6efaf3bf271fc1838613fa0778')

    @with_ephemeral_container(
        distributions=['ubuntu-vivid', 'ubuntu-trusty', 'centos-7'])
    def test_disable_requiretty_on_sshd_config_returns_True(self,
                                                            *args,
                                                            **kwargs):
        self.assertTrue(os_helpers.disable_requiretty_on_sshd_config())

    @with_ephemeral_container(
        distributions=['ubuntu-vivid', 'ubuntu-trusty', 'centos-7'])
    def test_disable_requiretty_on_sshd_config_raises_exception_on_failure(
            self, *args, **kwargs):
        # this will make if fail
        sudo('rm /etc/ssh/sshd_config')
        with self.assertRaises(SystemExit) as cm:
            os_helpers.disable_requiretty_on_sshd_config()
        self.assertEqual(cm.exception.code, 1)


class DisableSelinuxTests(DockerBasedTests):

    @unittest.skip("Unable to test SElinux on a container")
    @with_ephemeral_container(distributions=['centos-7'])
    def test_disable_selinux_disables_selinux(self, *args, **kwargs):
        self.assertTrue(False)

    @unittest.skip("Unable to test SElinux on a container")
    @with_ephemeral_container(distributions=['centos-7'])
    def test_add_disable_selinux_returns_True_on_sucess(self, *args, **kwargs):
        self.assertTrue(False)

    @unittest.skip("Unable to test SElinux on a container")
    @with_ephemeral_container(distributions=['centos-7'])
    def test_disable_selinux_raises_exception_on_failure(self, *args, **kwargs):
        self.assertTrue(False)

        with self.assertRaises(SystemExit) as cm:
            self.assertTrue(True)
        self.assertEqual(cm.exception.code, 1)


class EnableSelinuxTests(DockerBasedTests):

    @unittest.skip("Unable to test SElinux on a container")
    @with_ephemeral_container(distributions=['centos-7'])
    def test_enable_selinux_enables_selinux(self, *args, **kwargs):
        self.assertTrue(False)

    @unittest.skip("Unable to test SElinux on a container")
    @with_ephemeral_container(distributions=['centos-7'])
    def test_add_enable_selinux_returns_True_on_sucess(self, *args, **kwargs):
        self.assertTrue(False)

    @unittest.skip("Unable to test SElinux on a container")
    @with_ephemeral_container(distributions=['centos-7'])
    def test_enable_selinux_raises_exception_on_failure(self, *args, **kwargs):
        with self.assertRaises(SystemExit) as cm:
            self.assertTrue(True)
        self.assertEqual(cm.exception.code, 1)


class FileAttribsTests(DockerBasedTests):
    @with_ephemeral_container(
        distributions=['ubuntu-vivid', 'ubuntu-trusty', 'centos-7'])
    def test_file_attribs_with_sudo_updates_perms(self, *args, **kwargs):
        # make sure we have a user1 and group1 on our docker instance
        sudo('/usr/sbin/useradd user1')
        sudo('/usr/sbin/groupadd group1')
        sudo('echo > /tmp/temp1')

        os_helpers.file_attribs(location='/tmp/temp1',
                                         mode='755',
                                         owner='user1',
                                         group='group1',
                                         recursive=False,
                                         use_sudo=True)

        perms, x, owner, group = sudo('ls -l /tmp/temp1').split(' ')[0:4]
        self.assertEquals(perms, '-rwxr-xr-x')
        self.assertEquals(owner, 'user1')
        self.assertEquals(group, 'group1')

    @with_ephemeral_container(
        distributions=['ubuntu-vivid', 'ubuntu-trusty', 'centos-7'])
    def test_file_attribs_with_sudo_updates_perms_recursively(self,
                                                              *args,
                                                              **kwargs):
        # make sure we have a user1 and group1 on our docker instance
        sudo('/usr/sbin/useradd user1')
        sudo('/usr/sbin/groupadd group1')
        sudo('mkdir /tmp/temp1')
        sudo('echo > /tmp/temp1/file1')

        os_helpers.file_attribs(location='/tmp/temp1',
                                         mode='755',
                                         owner='user1',
                                         group='group1',
                                         recursive=True,
                                         use_sudo=True)

        perms, x, owner, group = sudo('ls -ld /tmp/temp1').split(' ')[0:4]
        self.assertEquals(perms, 'drwxr-xr-x')
        self.assertEquals(owner, 'user1')
        self.assertEquals(group, 'group1')

        perms, x, owner, group = sudo('ls -l /tmp/temp1/file1').split(' ')[0:4]
        self.assertEquals(perms, '-rwxr-xr-x')
        self.assertEquals(owner, 'user1')
        self.assertEquals(group, 'group1')

    @with_ephemeral_container(
        distributions=['ubuntu-vivid', 'ubuntu-trusty', 'centos-7'])
    def test_file_attribs_without_sudo_updates_perms(self, *args, **kwargs):
        # make sure we have a user1 and group1 on our docker instance
        sudo('/usr/sbin/useradd user1')
        sudo('/usr/sbin/groupadd group1')
        sudo('echo > /tmp/temp1')

        os_helpers.file_attribs(location='/tmp/temp1',
                                         mode='755',
                                         owner='user1',
                                         group='group1',
                                         recursive=False,
                                         use_sudo=False)

        perms, x, owner, group = sudo('ls -l /tmp/temp1').split(' ')[0:4]
        self.assertEquals(perms, '-rwxr-xr-x')
        self.assertEquals(owner, 'user1')
        self.assertEquals(group, 'group1')

    @with_ephemeral_container(
        distributions=['ubuntu-vivid', 'ubuntu-trusty', 'centos-7'])
    def test_file_attribs_without_sudo_updates_perms_recursively(self,
                                                                 *args,
                                                                 **kwargs):
        # make sure we have a user1 and group1 on our docker instance
        sudo('/usr/sbin/useradd user1')
        sudo('/usr/sbin/groupadd group1')
        sudo('mkdir /tmp/temp1')
        sudo('echo > /tmp/temp1/file1')

        os_helpers.file_attribs(location='/tmp/temp1',
                                         mode='755',
                                         owner='user1',
                                         group='group1',
                                         recursive=True,
                                         use_sudo=False)
        perms, x, owner, group = sudo('ls -ld /tmp/temp1').split(' ')[0:4]
        self.assertEquals(perms, 'drwxr-xr-x')
        self.assertEquals(owner, 'user1')
        self.assertEquals(group, 'group1')

        perms, x, owner, group = sudo('ls -l /tmp/temp1/file1').split(' ')[0:4]
        self.assertEquals(perms, '-rwxr-xr-x')
        self.assertEquals(owner, 'user1')
        self.assertEquals(group, 'group1')

    @with_ephemeral_container(
        distributions=['ubuntu-vivid', 'ubuntu-trusty', 'centos-7'])
    def test_file_attribs_returns_True_on_sucess(self, *args, **kwargs):
        # make sure we have a user1 and group1 on our docker instance
        sudo('/usr/sbin/useradd user1')
        sudo('/usr/sbin/groupadd group1')
        sudo('mkdir /tmp/temp1')
        sudo('echo > /tmp/temp1/file1')

        result = os_helpers.file_attribs(location='/tmp/temp1',
                                         mode='755',
                                         owner='user1',
                                         group='group1',
                                         recursive=True,
                                         use_sudo=True)

        self.assertTrue(result)

    @with_ephemeral_container(
        distributions=['ubuntu-vivid', 'ubuntu-trusty', 'centos-7'])
    def test_file_attribs_raises_exception_on_failure(self,
                                                      *args,
                                                      **kwargs):
        with self.assertRaises(SystemExit) as cm:
            os_helpers.file_attribs(location='/tmp/no-valid-path',
                                             mode='755',
                                             owner='not-a-valid-user',
                                             group='not-a-valid-group',
                                             recursive=True,
                                             use_sudo=False)
        self.assertEqual(cm.exception.code, 1)


class OsReleaseTests(DockerBasedTests):

    @with_ephemeral_container(
        distributions=['ubuntu-vivid'])
    def test_os_release_on_ubuntu_vivid_returns_os_release_string(self,
                                                                  *args,
                                                                  **kwargs):
        print(os_helpers.os_release())
        self.assertEquals(os_helpers.os_release(),
                          {'SUPPORT_URL': 'http://help.ubuntu.com/',
                           'NAME': 'Ubuntu',
                           'ID_LIKE': 'debian',
                           'VERSION_ID': '15.04',
                           'BUG_REPORT_URL': 'http://bugs.launchpad.net/ubuntu/',  # noqa
                           'PRETTY_NAME': 'Ubuntu 15.04',
                           'VERSION': '15.04 (Vivid Vervet)',
                           'HOME_URL': 'http://www.ubuntu.com/',
                           'ID': 'ubuntu'}
        )

    @with_ephemeral_container(
        distributions=['ubuntu-trusty'])
    def test_os_release_on_ubuntu_trusty_returns_os_release_string(self,
                                                                   *args,
                                                                   **kwargs):
        self.assertEquals(os_helpers.os_release(),
                          {'SUPPORT_URL': 'http://help.ubuntu.com/',
                           'NAME': 'Ubuntu',
                           'ID_LIKE': 'debian',
                           'VERSION_ID': '14.04',
                           'BUG_REPORT_URL': 'http://bugs.launchpad.net/ubuntu/',  # noqa
                           'PRETTY_NAME': 'Ubuntu 14.04.3 LTS',
                           'VERSION': '14.04.3 LTS, Trusty Tahr',
                           'HOME_URL': 'http://www.ubuntu.com/',
                           'ID': 'ubuntu'}
        )

    @with_ephemeral_container(
        distributions=['centos-7'])
    def test_os_release_on_centos_7_returns_os_release_string(self,
                                                              *args,
                                                              **kwargs):
        self.assertEquals(os_helpers.os_release(),
                          {'NAME': 'CentOS Linux',
                           'ANSI_COLOR': '0;31',
                           'ID_LIKE': 'rhel fedora',
                           'VERSION_ID': '7',
                           'BUG_REPORT_URL': 'https://bugs.centos.org/',
                           'CENTOS_MANTISBT_PROJECT': 'CentOS-7',
                           'PRETTY_NAME': 'CentOS Linux 7 (Core)',
                           'VERSION': '7 (Core)',
                           'REDHAT_SUPPORT_PRODUCT_VERSION': '7',
                           'CENTOS_MANTISBT_PROJECT_VERSION': '7',
                           'REDHAT_SUPPORT_PRODUCT': 'centos',
                           'HOME_URL': 'https://www.centos.org/',
                           'CPE_NAME': 'cpe:/o:centos:centos:7',
                           'ID': 'centos'})


class LinuxDistributionTests(DockerBasedTests):

    @with_ephemeral_container(
        distributions=['centos-7'])
    def test_linux_distribution_on_centos_returns_centos_string(self,
                                                                *args,
                                                                **kwargs):
        self.assertTrue(os_helpers.linux_distribution(), 'centos')

    @with_ephemeral_container(
        distributions=['ubuntu-trusty'])
    def test_linux_distribution_on_ubuntu_trusty_returns_ubuntu_string(
            self, *args, **kwargs):
        self.assertTrue(os_helpers.linux_distribution(), 'ubuntu')

    @with_ephemeral_container(
        distributions=['ubuntu-vivid'])
    def test_linux_distribution_on_ubuntu_vivid_returns_ubuntu_string(
            self, *args, **kwargs):
        self.assertTrue(os_helpers.linux_distribution(), 'ubuntu')


class LsbReleaseTests(DockerBasedTests):
    @with_ephemeral_container(
        distributions=['ubuntu-vivid'])
    def test_lsb_release_on_ubuntu_vivid_returns_lsb_release_string(self,
                                                                    *args,
                                                                    **kwargs):
        sudo('apt-get update')
        sudo('apt-get -y install lsb-release')
        self.assertEquals(os_helpers.lsb_release(),
                          {'DISTRIB_CODENAME': 'vivid',
                           'DISTRIB_RELEASE': '15.04',
                           'DISTRIB_ID': 'Ubuntu',
                           'DISTRIB_DESCRIPTION': 'Ubuntu 15.04'}
                          )

    @with_ephemeral_container(
        distributions=['ubuntu-trusty'])
    def test_lsb_release_on_ubuntu_trusty_returns_lsb_release_string(self,
                                                                     *args,
                                                                     **kwargs):
        sudo('apt-get update')
        sudo('apt-get -y install lsb-release')
        self.assertEquals(os_helpers.lsb_release(),
                          {'DISTRIB_CODENAME': 'trusty',
                           'DISTRIB_RELEASE': '14.04',
                           'DISTRIB_ID': 'Ubuntu',
                           'DISTRIB_DESCRIPTION': 'Ubuntu 14.04.3 LTS'}
                          )

    @unittest.skip("/etc/lsb-release not available on centos")
    @with_ephemeral_container(
        distributions=['centos-7'], verbose=True)
    def test_lsb_release_on_centos_7_lsb_release_string(self,
                                                        *args,
                                                        **kwargs):
        sudo('yum install -y redhat-lsb-core')
        self.assertEquals(os_helpers.lsb_release(),
                          {'DISTRIB_CODENAME': 'trusty',
                           'DISTRIB_RELEASE': '14.04',
                           'DISTRIB_ID': 'Ubuntu',
                           'DISTRIB_DESCRIPTION': 'Ubuntu 14.04.3 LTS'}
                          )


class RebootTests(DockerBasedTests):

    @unittest.skip("Unable to test a system reboot on a container")
    @with_ephemeral_container(
        distributions=['ubuntu-vivid', 'ubuntu-trusty', 'centos-7'])
    def test_reboot_returns_reboot_string(self, *args, **kwargs):
        self.assertTrue(False)


class RestartServiceTests(DockerBasedTests):

    @with_ephemeral_container(
        distributions=['ubuntu-vivid', 'ubuntu-trusty'])
    def test_restart_service__on_ubuntu_restarts_service(self, *args, **kwargs):
        sudo('apt-get update')
        sudo('apt-get install -y xinetd')
        sudo('service xinetd start')

        pid_before_restart = sudo(
            'ps -ef | grep xinetd | grep -v grep | awk "{ print $2 }"')
        os_helpers.restart_service('xinetd')
        pid_after_restart = sudo(
            'ps -ef | grep xinetd | grep -v grep | awk "{ print $2 }"')
        self.assertNotEquals(pid_before_restart, pid_after_restart)

    @unittest.skip("Required systemd running on the container")
    @with_ephemeral_container(
        distributions=['centos-7'])
    def test_restart_service__on_centos_restarts_service(self, *args, **kwargs):
        sudo('yum -y install xinetd')
        sudo('service xinetd start')

        pid_before_restart = sudo(
            'ps -ef | grep xinetd | grep -v grep | awk "{ print $2 }"')
        os_helpers.restart_service('xinetd')
        pid_after_restart = sudo(
            'ps -ef | grep xinetd | grep -v grep | awk "{ print $2 }"')
        self.assertNotEquals(pid_before_restart, pid_after_restart)

    @with_ephemeral_container(
        distributions=['ubuntu-vivid', 'ubuntu-trusty'])
    def test_restart_service_on_ubuntu_returns_True_on_success(self,
                                                               *args,
                                                               **kwargs):
        sudo('apt-get update')
        sudo('apt-get install -y xinetd')
        sudo('service xinetd start')

        self.assertTrue(os_helpers.restart_service('xinetd'))

    @unittest.skip("Required systemd running on the container")
    @with_ephemeral_container(
        distributions=['centos-7'])
    def test_restart_service_on_centos_returns_True_on_success(self,
                                                               *args,
                                                               **kwargs):
        sudo('apt-get update')
        sudo('apt-get install -y xinetd')
        sudo('service xinetd start')

        self.assertTrue(os_helpers.restart_service('xinetd'))

    @with_ephemeral_container(
        distributions=['ubuntu-vivid', 'ubuntu-trusty'])
    def test_restart_service_on_ubuntu_raises_exception_on_failure(self,
                                                                   *args,
                                                                   **kwargs):
        with self.assertRaises(SystemExit) as cm:
            os_helpers.restart_service('fake-service')
        self.assertEqual(cm.exception.code, 1)

    @unittest.skip("Required systemd running on the container")
    @with_ephemeral_container(
        distributions=['centos-7'])
    def test_restart_service_on_centos_raises_exception_on_failure(self,
                                                                   *args,
                                                                   **kwargs):
        with self.assertRaises(SystemExit) as cm:
            os_helpers.restart_service('fake-service')
        self.assertEqual(cm.exception.code, 1)


class SystemdTests(DockerBasedTests):

    @unittest.skip("Required systemd running on the container")
    @with_ephemeral_container(distributions=['centos-7'])
    def test_systemd_returns_systemd_string(self, *args, **kwargs):
        self.assertTrue(False)

    @unittest.skip("Required systemd running on the container")
    @with_ephemeral_container(distributions=['centos-7'])
    def test_systemd_returns_True_on_sucess(self, *args, **kwargs):
        self.assertTrue(False)

    @unittest.skip("Required systemd running on the container")
    @with_ephemeral_container(distributions=['centos-7'])
    def test_systemd_raises_exception_on_failure(self, *args, **kwargs):
        self.assertTrue(False)

        with self.assertRaises(SystemExit) as cm:
            self.assertTrue(True)
        self.assertEqual(cm.exception.code, 1)


class AddFirewalldServiceTests(DockerBasedTests):

    @unittest.skip("Required systemd running on the container")
    @with_ephemeral_container(distributions=['centos-7'])
    def test_add_firewalld_service_adds_service(self, *args, **kwargs):
        self.assertTrue(False)

    @unittest.skip("Required systemd running on the container")
    @with_ephemeral_container(distributions=['centos-7'])
    def test_add_frewalld_service_returns_True_on_sucess(self, *args, **kwargs):
        self.assertTrue(False)

    @unittest.skip("Required systemd running on the container")
    @with_ephemeral_container(distributions=['centos-7'])
    def test_add_firewalld_service_raises_exception_on_failure(self,
                                                               *args,
                                                               **kwargs):
        self.assertTrue(False)

        with self.assertRaises(SystemExit) as cm:
            self.assertTrue(True)
        self.assertEqual(cm.exception.code, 1)


class AddFirewalldPortTests(DockerBasedTests):

    @unittest.skip("Required systemd running on the container")
    @with_ephemeral_container(distributions=['centos-7'])
    def test_add_firewalld_port_adds_port(self, *args, **kwargs):
        self.assertTrue(False)

    @unittest.skip("Required systemd running on the container")
    @with_ephemeral_container(distributions=['centos-7'])
    def test_add_firewalld_port_returns_True_on_sucess(self, *args, **kwargs):
        self.assertTrue(False)

    @unittest.skip("Required systemd running on the container")
    @with_ephemeral_container(distributions=['centos-7'])
    def test_add_firewalld_port_raises_exception_on_failure(self,
                                                            *args,
                                                            **kwargs):
        self.assertTrue(False)

        with self.assertRaises(SystemExit) as cm:
            self.assertTrue(True)
        self.assertEqual(cm.exception.code, 1)


class EnableFirewalldServiceTests(DockerBasedTests):

    @unittest.skip("Required systemd running on the container")
    @with_ephemeral_container(distributions=['centos-7'])
    def test_enable_firewalld_service_enables_service(self, *args, **kwargs):
        self.assertTrue(False)

    @unittest.skip("Required systemd running on the container")
    @with_ephemeral_container(distributions=['centos-7'])
    def test_enable_firewalld_service_returns_True_on_sucess(self,
                                                             *args,
                                                             **kwargs):
        self.assertTrue(False)

    @unittest.skip("Required systemd running on the container")
    @with_ephemeral_container(distributions=['centos-7'])
    def test_enable_firewalld_service_raises_exception_on_failure(self,
                                                                  *args,
                                                                  **kwargs):
        self.assertTrue(False)

        with self.assertRaises(SystemExit) as cm:
            self.assertTrue(True)
        self.assertEqual(cm.exception.code, 1)


if __name__ == '__main__':
    unittest.main()



# vim: ai ts=4 sts=4 et sw=4 ft=python fdm=indent et foldlevel=0
import uuid
import os
import re

from fabric.operations import (get as get_file,
                               put as upload_file)


def insert_line_in_file_after_regex(path, line, after_regex, use_sudo=False):
    """ inserts a line in the middle of a file """

    tmpfile = str(uuid.uuid4())
    get_file(path, tmpfile, use_sudo=use_sudo)
    with open(tmpfile) as f:
        original = f.read()

    has_it_changed = False
    if line not in original:
        has_it_changed = True
        outfile = str(uuid.uuid4())
        with open(outfile, 'w') as output:
            for l in original.split('\n'):
                output.write(l + '\n')
                if re.match(after_regex, l) is not None:
                    output.write(line + '\n')

        upload_file(local_path=outfile,
                    remote_path=path,
                    use_sudo=use_sudo)
        os.unlink(outfile)
    os.unlink(tmpfile)

    return has_it_changed



import unittest
from bookshelf.api_v2 import rackspace
import novaclient
import vcr
import mock
from collections import OrderedDict
import os



class ConnectToRackspaceTests(unittest.TestCase):

    @vcr.use_cassette('fixtures/vcr_ConnectToRackspace.yml',
                      decode_compressed_response=True)
    def test_connect_to_rackspace_returns_connection_object(self,
                                                            *args,
                                                            **kwargs):

        conn = rackspace.connect_to_rackspace(os.environ.get('OS_REGION_NAME'),
                                              os.environ.get('OS_USERNAME'),
                                              os.environ.get('OS_PASSWORD'),
                                              )
        self.assertTrue(isinstance(conn, novaclient.v2.client.Client))


class CreateRackspaceImageTests(unittest.TestCase):
    @vcr.use_cassette('fixtures/vcr_CreateRackspaceImage.yml')
    def test_CreateRackspaceImage(self,
                                  *args,
                                  **kwargs):

        conn = rackspace.connect_to_rackspace(os.environ.get('OS_REGION_NAME'),
                                              os.environ.get('OS_USERNAME'),
                                              os.environ.get('OS_PASSWORD'),
                                              )

        # mock. wait for ssh
        import bookshelf.api_v2.cloud
        bookshelf.api_v2.cloud.is_ssh_available = mock.Mock(return_value=True)

        srv = rackspace.create_server_rackspace(connection=conn,
                                                distribution='centos7',
                                                disk_name='disk',
                                                disk_size='75',
                                                ami='CentOS 7 (PVHVM)',
                                                region='IAD',
                                                key_pair='mykeypair',
                                                instance_type='1GB Standard Instance', # noqa
                                                instance_name='CentOS_7',
                                                tags={},
                                                security_groups=None)

        img = rackspace.create_rackspace_image(connection=conn,
                                               name='myImage',
                                               description='my Image',
                                               server_id=srv.id)

        self.assertRegexpMatches(img, '^.*-.*-.*-.*-*$')


class CreateServerRackspaceTests(unittest.TestCase):

    @vcr.use_cassette('fixtures/vcr_CreateServerRackspace.yml')
    def test_CreateServerRackspace(self,
                                   *args,
                                   **kwargs):

        conn = rackspace.connect_to_rackspace(os.environ.get('OS_REGION_NAME'),
                                              os.environ.get('OS_USERNAME'),
                                              os.environ.get('OS_PASSWORD'),
                                              )
        # mock. wait for ssh
        import bookshelf.api_v2.cloud
        bookshelf.api_v2.cloud.is_ssh_available = mock.Mock(return_value=True)

        srv = rackspace.create_server_rackspace(connection=conn,
                                                distribution='centos7',
                                                disk_name='disk',
                                                disk_size='75',
                                                ami='CentOS 7 (PVHVM)',
                                                region='IAD',
                                                key_pair='mykeypair',
                                                instance_type='1GB Standard Instance', # noqa
                                                instance_name='CentOS_7',
                                                tags={},
                                                security_groups=None)

        self.assertIsInstance(srv,
                              novaclient.v2.servers.Server)


class DestroyRackspaceTests(unittest.TestCase):

    @vcr.use_cassette('fixtures/vcr_DestroyRackspace.yml')
    def test_DestroyRackspace(self,
                              *args,
                              **kwargs):

        conn = rackspace.connect_to_rackspace(os.environ.get('OS_REGION_NAME'),
                                              os.environ.get('OS_USERNAME'),
                                              os.environ.get('OS_PASSWORD'),
                                              )

        # mock. wait for ssh
        import bookshelf.api_v2.cloud
        bookshelf.api_v2.cloud.is_ssh_available = mock.Mock(return_value=True)

        srv = rackspace.create_server_rackspace(connection=conn,
                                                distribution='centos7',
                                                disk_name='disk',
                                                disk_size='75',
                                                ami='CentOS 7 (PVHVM)',
                                                region='IAD',
                                                key_pair='mykeypair',
                                                instance_type='1GB Standard Instance',
                                                instance_name='CentOS_7',
                                                tags={},
                                                security_groups=None)

        rackspace.destroy_rackspace(connection=conn,
                                    region='IAD',
                                    instance_id=srv.id)


class DownRackspaceTests(unittest.TestCase):

    @vcr.use_cassette('fixtures/vcr_DownRackspace.yml')
    def test_DownRackspace(self,
                              *args,
                              **kwargs):

        conn = rackspace.connect_to_rackspace(os.environ.get('OS_REGION_NAME'),
                                              os.environ.get('OS_USERNAME'),
                                              os.environ.get('OS_PASSWORD'),
                                              )

        # mock. wait for ssh
        import bookshelf.api_v2.cloud
        bookshelf.api_v2.cloud.is_ssh_available = mock.Mock(return_value=True)

        srv = rackspace.create_server_rackspace(connection=conn,
                                                distribution='centos7',
                                                disk_name='disk',
                                                disk_size='75',
                                                ami='CentOS 7 (PVHVM)',
                                                region='IAD',
                                                key_pair='mykeypair',
                                                instance_type='1GB Standard Instance',
                                                instance_name='CentOS_7',
                                                tags={},
                                                security_groups=None)

        rackspace.destroy_rackspace(connection=conn,
                                    region='IAD',
                                    instance_id=srv.id)


class GetIpAddressFromRackspaceServerTests(unittest.TestCase):

    @vcr.use_cassette('fixtures/vcr_GetIpAddressFromRackspaceServer.yml')
    def test_GetIpAdressFromRackspaceServer(self,
                              *args,
                              **kwargs):

        conn = rackspace.connect_to_rackspace(os.environ.get('OS_REGION_NAME'),
                                              os.environ.get('OS_USERNAME'),
                                              os.environ.get('OS_PASSWORD'),
                                              )
        # mock. wait for ssh
        import bookshelf.api_v2.cloud
        bookshelf.api_v2.cloud.is_ssh_available = mock.Mock(return_value=True)

        srv = rackspace.create_server_rackspace(connection=conn,
                                                distribution='centos7',
                                                disk_name='disk',
                                                disk_size='75',
                                                ami='CentOS 7 (PVHVM)',
                                                region='IAD',
                                                key_pair='mykeypair',
                                                instance_type='1GB Standard Instance',
                                                instance_name='CentOS_7',
                                                tags={},
                                                security_groups=None)

        self.assertRegexpMatches(
            rackspace.get_ip_address_from_rackspace_server(connection=conn,
                                                           server_id=srv.id
                                                           ),
            '\d+\.\d+\.\d+\.\d+'
        )


class GetRackspaceInfoTests(unittest.TestCase):

    @vcr.use_cassette('fixtures/vcr_RackspaceInfo.yml')
    def test_RackspaceInfo(self,
                           *args,
                           **kwargs):

        def sortOD(od):
            res = OrderedDict()
            for k, v in sorted(od.items()):
                if isinstance(v, dict):
                    res[k] = sortOD(v)
                else:
                    res[k] = v
            return res

        conn = rackspace.connect_to_rackspace(os.environ.get('OS_REGION_NAME'),
                                              os.environ.get('OS_USERNAME'),
                                              os.environ.get('OS_PASSWORD'),
                                              )
        # mock. wait for ssh
        import bookshelf.api_v2.cloud
        bookshelf.api_v2.cloud.is_ssh_available = mock.Mock(return_value=True)

        srv = rackspace.create_server_rackspace(connection=conn,
                                                distribution='centos7',
                                                disk_name='disk',
                                                disk_size='75',
                                                ami='CentOS 7 (PVHVM)',
                                                region='IAD',
                                                key_pair='mykeypair',
                                                instance_type='1GB Standard Instance',
                                                instance_name='CentOS_7',
                                                tags={},
                                                security_groups=None)

        data = rackspace.get_rackspace_info(connection=conn,
                                            server_id=srv.id)

        expected_keys = [
            'ip_address',
            'accessIPv4',
            'accessIPv6',
            'addresses',
            'created',
            'flavor',
            'id',
            'human_id',
            'image',
            'key_name',
            'state',
            'metadata',
            'name',
            'networks',
            'tenant_id',
            'user_id',
            'cloud_type'
        ]


        for k in expected_keys:
            self.assertTrue(k in data)


if __name__ == '__main__':
    unittest.main()



# vim: ai ts=4 sts=4 et sw=4 ft=python fdm=indent et foldlevel=0

from fabric.api import sudo, settings, run, hide


def update_system_pip_to_latest_pip():
    """ install the latest pip """
    sudo("pip install --quiet --upgrade pip")


def update_to_latest_pip():
    """ install the latest pip """
    run("pip install --quiet --upgrade pip")



# vim: ai ts=4 sts=4 et sw=4 ft=python fdm=indent et foldlevel=0

from fabric.api import sudo, settings, run
from fabric.context_managers import hide
from fabric.contrib.files import append as file_append
from fabric.contrib.files import comment as comment_line
from fabric.contrib.files import sed, contains

import bookshelf.api_v2 as bookshelf2


def add_usr_local_bin_to_path(log=False):
    """ adds /usr/local/bin to $PATH """
    if log:
        bookshelf2.logging_helpers.log_green('inserts /usr/local/bin into PATH')

    with settings(hide('warnings', 'running', 'stdout', 'stderr'),
                  capture=True):
        try:
            sudo('echo "export PATH=/usr/local/bin:$PATH" '
                 '|sudo /usr/bin/tee /etc/profile.d/fix-path.sh')
            return True
        except:
            raise SystemExit(1)


def arch():
    """ returns the current cpu archictecture """
    with settings(hide('warnings', 'running', 'stdout', 'stderr'),
                  warn_only=True, capture=True):
        result = sudo('uname --machine').strip()
    return result


def dir_attribs(location, mode=None, owner=None,
                group=None, recursive=False, use_sudo=False):
    """ cuisine dir_attribs doesn't do sudo, so we implement our own
        Updates the mode/owner/group for the given remote directory."""
    args = ''
    if recursive:
        args = args + ' -R '

    if mode:
        if use_sudo:
            sudo('chmod %s %s %s' % (args, mode,  location))
        else:
            run('chmod %s %s %s' % (args, mode,  location))
    if owner:
        if use_sudo:
            sudo('chown %s %s %s' % (args, owner, location))
        else:
            run('chown %s %s %s' % (args, owner, location))
    if group:
        if use_sudo:
            sudo('chgrp %s %s %s' % (args, group, location))
        else:
            run('chgrp %s %s %s' % (args, group, location))
    return True


def dir_ensure(location, recursive=False, mode=None,
               owner=None, group=None, use_sudo=False):
    """ cuisine dir_ensure doesn't do sudo, so we implement our own
    Ensures that there is a remote directory at the given location,
    optionally updating its mode/owner/group.
    If we are not updating the owner/group then this can be done as a single
    ssh call, so use that method, otherwise set owner/group after creation."""

    args = ''
    if recursive:
        args = args + ' -p '

    if not dir_exists(location):
        if use_sudo:
            sudo('mkdir %s %s' % (args, location))
        else:
            run('mkdir %s %s' % (args, location))

    if owner or group or mode:
        if use_sudo:
            dir_attribs(location,
                        owner=owner,
                        group=group,
                        mode=mode,
                        recursive=recursive,
                        use_sudo=True)
        else:
            dir_attribs(location,
                        owner=owner,
                        group=group,
                        mode=mode,
                        recursive=recursive)
    return True


def dir_exists(location, use_sudo=False):
    """Tells if there is a remote directory at the given location."""
    with settings(hide('running', 'stdout', 'stderr'), warn_only=True):
        if use_sudo:
            # convert return code 0 to True
            return not bool(sudo('test -d %s' % (location)).return_code)
        else:
            return not bool(run('test -d %s' % (location)).return_code)


def disable_env_reset_on_sudo(log=False):
    """ updates /etc/sudoers so that users from %wheel keep their
        environment when executing a sudo call
    """
    if log:
        bookshelf2.logging_helpers.log_green('disabling env reset on sudo')

    file_append('/etc/sudoers',
                'Defaults:%wheel !env_reset,!secure_path',
                use_sudo=True,
                partial=True)
    return True


def disable_requiretty_on_sudoers(log=False):
    """ allow sudo calls through ssh without a tty """
    if log:
        bookshelf2.logging_helpers.log_green(
            'disabling requiretty on sudo calls')

    comment_line('/etc/sudoers',
                 '^Defaults.*requiretty', use_sudo=True)
    return True


def disable_requiretty_on_sshd_config(log=False):
    """ allow sudo calls through ssh without a tty """
    if log:
        bookshelf2.logging_helpers.log_green(
            'disabling requiretty on sshd_config')

    comment_line('/etc/ssh/sshd_config',
                 '^Defaults.*requiretty', use_sudo=True)
    return True


def disable_selinux():
    """ disables selinux """

    if contains(filename='/etc/selinux/config',
                text='SELINUX=enforcing'):
        sed('/etc/selinux/config',
            'SELINUX=enforcing', 'SELINUX=disabled', use_sudo=True)

    if contains(filename='/etc/selinux/config',
                text='SELINUX=permissive'):
        sed('/etc/selinux/config',
            'SELINUX=permissive', 'SELINUX=disabled', use_sudo=True)

    if sudo('getenforce').lower() != 'disabled':
        with settings(warn_only=True, capture=True):
            sudo('/sbin/reboot')
        bookshelf2.time_helpers.sleep_for_one_minute()


def enable_selinux():
    """ disables selinux """

    if not contains(filename='/etc/selinux/config',
                    text='SELINUX=enforcing'):
        sed('/etc/selinux/config',
            'SELINUX=.*', 'SELINUX=enforcing', use_sudo=True)

    if contains(filename='/etc/selinux/config',
                text='SELINUXTYPE=targeted'):
        sed('/etc/selinux/config',
            'SELINUXTYPE=.*', 'SELINUX=targeted', use_sudo=True)

    sudo('/sbin/setenforce 1')

    if sudo('getenforce') != 'Enforcing':
        with settings(warn_only=True, capture=True):
            sudo('/sbin/reboot')
        bookshelf2.time_helpers.sleep_for_one_minute()


def file_attribs(location,
                 mode=None,
                 owner=None,
                 group=None,
                 use_sudo=False,
                 recursive=True):
    """Updates the mode/owner/group for the remote file at the given
    location."""
    return dir_attribs(location=location,
                       mode=mode,
                       owner=owner,
                       group=group,
                       recursive=recursive,
                       use_sudo=False)


def os_release():
    """ returns /etc/os-release in a dictionary """
    with settings(hide('warnings', 'running', 'stderr'),
                  warn_only=True, capture=True):

        release = {}
        data = run('cat /etc/os-release')
        for line in data.split('\n'):
            if not line:
                continue
            parts = line.split('=')
            if len(parts) == 2:
                release[parts[0]] = parts[1].strip('\n\r"')

        return release


def linux_distribution():
    """ returns the linux distribution in lower case """
    with settings(hide('warnings', 'running', 'stdout', 'stderr'),
                  warn_only=True, capture=True):
        data = os_release()
        return(data['ID'])


def lsb_release():
    """ returns /etc/lsb-release in a dictionary """
    with settings(hide('warnings', 'running'), capture=True):

        _lsb_release = {}
        data = sudo('cat /etc/lsb-release')
        for line in data.split('\n'):
            if not line:
                continue
            parts = line.split('=')
            if len(parts) == 2:
                _lsb_release[parts[0]] = parts[1].strip('\n\r"')

        return _lsb_release


def reboot():

    with settings(warn_only=True, capture=True):
        sudo('shutdown -r now')
        bookshelf2.time_helpers.sleep_for_one_minute()


def restart_service(service, log=False):
    """ restarts a service  """
    with settings():
        if log:
            bookshelf2.logging_helpers.log_yellow(
                'stoping service %s' % service)
        sudo('service %s stop' % service)
        if log:
            bookshelf2.logging_helpers.log_yellow(
                'starting service %s' % service)
        sudo('service %s start' % service)
    return True


def systemd(service, start=True, enabled=True, unmask=False, restart=False):
    """ manipulates systemd services """

    with settings(hide('warnings', 'running', 'stdout', 'stderr'),
                  warn_only=True, capture=True):

        if restart:
            sudo('systemctl restart %s' % service)
        else:
            if start:
                sudo('systemctl start %s' % service)
            else:
                sudo('systemctl stop %s' % service)

        if enabled:
            sudo('systemctl enable %s' % service)
        else:
            sudo('systemctl disable %s' % service)

        if unmask:
            sudo('systemctl unmask %s' % service)


def add_firewalld_service(service, permanent=True):
    """ adds a firewall rule """

    bookshelf2.pkg.yum_install(packages=['firewalld'])

    with settings(hide('warnings', 'running', 'stdout', 'stderr'),
                  warn_only=True, capture=True):
        p = ''
        if permanent:
            p = '--permanent'
        sudo('firewall-cmd --add-service %s %s' % (service, p))
        sudo('systemctl reload firewalld')


def add_firewalld_port(port, permanent=True):
    """ adds a firewall rule """

    bookshelf2.pkg.yum_install(packages=['firewalld'])

    bookshelf2.logging_helpers.log_green('adding a new fw rule: %s' % port)
    with settings(hide('warnings', 'running', 'stdout', 'stderr'),
                  warn_only=True, capture=True):
        p = ''
        if permanent:
            p = '--permanent'
        sudo('firewall-cmd --add-port %s %s' % (port, p))
        sudo('systemctl restart firewalld')


def enable_firewalld_service():
    """ install and enables the firewalld service """

    bookshelf2.pkg.yum_install(packages=['firewalld'])
    systemd(service='firewalld', unmask=True)


def install_os_updates(distribution, force=False):
    """ installs OS updates """
    if ('centos' in distribution or
            'rhel' in distribution or
            'redhat' in distribution):
        bookshelf2.logging_helpers.log_green('installing OS updates')
        sudo("yum -y --quiet clean all")
        sudo("yum group mark convert")
        sudo("yum -y --quiet update")

    if ('ubuntu' in distribution or
            'debian' in distribution):
        with settings(hide('warnings', 'running', 'stdout', 'stderr'),
                      warn_only=False, capture=True):
            sudo("DEBIAN_FRONTEND=noninteractive apt-get update")
            if force:
                sudo("sudo DEBIAN_FRONTEND=noninteractive apt-get -y -o "
                     "Dpkg::Options::='--force-confdef' "
                     "-o Dpkg::Options::='--force-confold' upgrade --force-yes")
            else:
                sudo("sudo DEBIAN_FRONTEND=noninteractive apt-get -y -o "
                     "Dpkg::Options::='--force-confdef' -o "
                     "Dpkg::Options::='--force-confold' upgrade")


def install_ubuntu_development_tools():
    """ installs development tools """

    bookshelf2.pkg.apt_install(packages=['build-essential'])


def install_centos_development_tools():
    """ installs development tools """

    bookshelf2.pkg.yum_group_install(groups=['Development tools'])



# from:
# http://stackoverflow.com/questions/16867347/step-by-step-debugging-with-ipython
# usage:
# from bookshelf.api_v2.debug import ipsh
# ipsh()


import inspect

# First import the embed function
from IPython.terminal.embed import InteractiveShellEmbed
from IPython.config.loader import Config

# Configure the prompt so that I know I am in a nested (embedded) shell
cfg = Config()
prompt_config = cfg.PromptManager
prompt_config.in_template = 'N.In <\\#>: '
prompt_config.in2_template = '   .\\D.: '
prompt_config.out_template = 'N.Out<\\#>: '

# Messages displayed when I drop into and exit the shell.
banner_msg = ("\n**Nested Interpreter:\n"
"Hit Ctrl-D to exit interpreter and continue program.\n"
"Note that if you use %kill_embedded, you can fully deactivate\n"
"This embedded instance so it will never turn on again")
exit_msg = '**Leaving Nested interpreter'

# Wrap it in a function that gives me more context:
def ipsh():
    ipshell = InteractiveShellEmbed(config=cfg, banner1=banner_msg, exit_msg=exit_msg)

    frame = inspect.currentframe().f_back
    msg   = 'Stopped at {0.f_code.co_filename} at line {0.f_lineno}'.format(frame)

    # Go back one level!
    # This is needed because the call to ipshell is inside the function ipsh()
    ipshell(msg,stack_depth=2)


# vim: ai ts=4 sts=4 et sw=4 ft=python fdm=indent et foldlevel=0

from fabric.api import sudo, run
from fabric.context_managers import cd
from fabric.contrib.files import exists


def install_recent_git_from_source(version='2.4.6',
                                   prefix='/usr/local',
                                   log=False):
    # update git
    sudo("wget -c https://www.kernel.org/pub/software/scm/git/git-%s.tar.gz" %
         version)
    sudo("test -e git-%s || tar -zxf git-%s.tar.gz" % (version, version))
    with cd('git-%s' % version):
        sudo('test -e %s/bin/git || ./configure --prefix=%s' % (prefix, prefix))
        sudo('test -e %s/bin/git || make' % prefix)
        sudo('test -e %s/bin/git || make install' % prefix)


def git_clone(repo_url, repo_name):
    """ clones a git repository """
    if not exists(repo_name):
        run("git clone %s" % repo_url)



# vim: ai ts=4 sts=4 et sw=4 ft=python fdm=indent et foldlevel=0

import pyrax
import re
import sys
from fabric.api import env
from sys import exit
from time import sleep
from bookshelf.api_v2.logging_helpers import log_green, log_yellow, log_red
from bookshelf.api_v2.time_helpers import sleep_for_one_minute
from bookshelf.api_v2.cloud import wait_for_ssh


def connect_to_rackspace(region,
                         access_key_id,
                         secret_access_key):
    """ returns a connection object to Rackspace  """
    pyrax.set_setting('identity_type', 'rackspace')
    pyrax.set_default_region(region)
    pyrax.set_credentials(access_key_id, secret_access_key)
    nova = pyrax.connect_to_cloudservers(region=region)
    return nova


def create_rackspace_image(connection,
                           server_id,
                           name,
                           description,
                           block_device_mapping=None):

    image_id = connection.servers.create_image(server_id, name)
    image = connection.images.get(image_id).status.lower()
    log_green('creating rackspace image...')
    while connection.images.get(image_id).status.lower() not in ['active',
                                                                 'error']:
        log_green('building rackspace image...')
        sleep_for_one_minute()

    if image == 'error':
        log_red('error creating image')
        sys.exit(1)

    log_green('finished image: %s' % image_id)
    return image_id


def create_server_rackspace(connection,
                            distribution,
                            disk_name,
                            disk_size,
                            ami,
                            region,
                            key_pair,
                            instance_type,
                            instance_name,
                            tags={},
                            security_groups=None):
    """
    Creates Rackspace Instance and saves it state in a local json file
    """

    log_yellow("Creating Rackspace instance...")

    flavor = connection.flavors.find(name=instance_type)
    image = connection.images.find(name=ami)

    server = connection.servers.create(name=instance_name,
                                       flavor=flavor.id,
                                       image=image.id,
                                       region=region,
                                       availability_zone=region,
                                       key_name=key_pair)

    while server.status == 'BUILD':
        log_yellow("Waiting for build to finish...")
        sleep(5)
        server = connection.servers.get(server.id)

    # check for errors
    if server.status != 'ACTIVE':
        log_red("Error creating rackspace instance")
        exit(1)

    # the server was assigned IPv4 and IPv6 addresses, locate the IPv4 address
    ip_address = server.accessIPv4

    if ip_address is None:
        log_red('No IP address assigned')
        exit(1)

    wait_for_ssh(ip_address)
    log_green('New server with IP address {0}.'.format(ip_address))
    return server


def destroy_rackspace(connection, region, instance_id):
    """ terminates the instance """

    server = connection.servers.get(instance_id)
    log_yellow('deleting rackspace instance ...')
    server.delete()

    # wait for server to be deleted
    try:
        while True:
            server = connection.servers.get(server.id)
            log_yellow('waiting for deletion ...')
            sleep(5)
    except:
        pass
    log_green('The server has been deleted')


def down_rackspace(connection, region, instance_id):
    """ terminates the instance """

    return(
        destroy_rackspace(connection, region, instance_id)
    )


def get_ip_address_from_rackspace_server(connection, server_id):
    """
    returns an ipaddress for a rackspace instance
    """
    server = connection.servers.get(server_id)
    # the server was assigned IPv4 and IPv6 addresses, locate the IPv4 address
    ip_address = None
    for network in server.networks['public']:
        if re.match('\d+\.\d+\.\d+\.\d+', network):
            ip_address = network
            break

    # find out if we have an ip address
    if ip_address is None:
        log_red('No IP address assigned')
        return False
    else:
        return ip_address


def get_rackspace_info(connection,
                       server_id):
    """ queries Rackspace for details about a particular server id
    """
    server = connection.servers.get(server_id)

    data = {}
    data['ip_address'] = server.accessIPv4
    data['accessIPv4'] = server.accessIPv4
    data['accessIPv6'] = server.accessIPv6
    data['addresses'] = server.addresses
    data['created'] = server.created
    data['flavor'] = server.flavor
    data['id'] = server.hostId
    data['human_id'] = server.human_id
    data['image'] = server.image['id']
    data['key_name'] = server.key_name
    data['state'] = server.status
    data['metadata'] = server.metadata
    data['name'] = server.name
    data['networks'] = server.networks
    data['tenant_id'] = server.tenant_id
    data['user_id'] = server.user_id
    data['cloud_type'] = 'rackspace'
    return data


def rackspace():
    env.cloud = 'rackspace'
    env.user = 'root'


def up_rackspace(region,
                 access_key_id,
                 secret_access_key,
                 instance_id,
                 username):
    """ boots an existing rackspace instance, or creates a new one if needed """
    # if we don't have a state file, then its likely we need to create a new
    # rackspace instance.
    log_red('not implemented')
    exit(1)



# vim: ai ts=4 sts=4 et sw=4 ft=python fdm=indent et foldlevel=0

import socket
from time import sleep

from bookshelf.api_v2.logging_helpers import log_yellow


def is_ssh_available(host, port=22):
    """ checks if ssh port is open """
    s = socket.socket()
    try:
        s.connect((host, port))
        return True
    except:
        return False


def wait_for_ssh(host, port=22, timeout=600):
    """ probes the ssh port and waits until it is available """
    log_yellow('waiting for ssh...')
    for iteration in xrange(1, timeout): #noqa
        sleep(1)
        if is_ssh_available(host, port):
            return True
        else:
            log_yellow('waiting for ssh...')



# vim: ai ts=4 sts=4 et sw=4 ft=python fdm=indent et

import boto.ec2

from boto.ec2.blockdevicemapping import BlockDeviceMapping, EBSBlockDeviceType
from fabric.api import env
from time import sleep
from bookshelf.api_v2.time_helpers import sleep_for_one_minute
from bookshelf.api_v2.logging_helpers import log_green, log_yellow, log_red
from bookshelf.api_v2.cloud import wait_for_ssh


def connect_to_ec2(region, access_key_id, secret_access_key):
    """ returns a connection object to AWS EC2  """
    conn = boto.ec2.connect_to_region(region,
                                      aws_access_key_id=access_key_id,
                                      aws_secret_access_key=secret_access_key)
    if conn:
        return conn
    else:
        return False


def create_ami(connection,
               region,
               instance_id,
               name,
               description,
               block_device_mapping=None,
               log=False):
    ami = connection.create_image(instance_id,
                                  name,
                                  description,
                                  block_device_mapping)

    image_status = connection.get_image(ami)
    while (image_status.state != "available" and
           image_status.state != "failed"):
        if log:
            log_yellow('creating ami...')
        sleep_for_one_minute()
        image_status = connection.get_image(ami)

    if image_status.state == "available":
        if log:
            log_green("ami %s %s" % (ami, image_status))
        return(ami)
    else:
        if log:
            log_red("ami %s %s" % (ami, image_status))
        return False


def create_server_ec2(connection,
                      region,
                      disk_name,
                      disk_size,
                      ami,
                      key_pair,
                      instance_type,
                      tags={},
                      security_groups=None,
                      delete_on_termination=True,
                      log=False,
                      wait_for_ssh_available=True):
    """
    Creates EC2 Instance
    """

    if log:
        log_green("Started...")
        log_yellow("...Creating EC2 instance...")

    ebs_volume = EBSBlockDeviceType()
    ebs_volume.size = disk_size
    bdm = BlockDeviceMapping()
    bdm[disk_name] = ebs_volume

    # get an ec2 ami image object with our choosen ami
    image = connection.get_all_images(ami)[0]
    # start a new instance
    reservation = image.run(1, 1,
                            key_name=key_pair,
                            security_groups=security_groups,
                            block_device_map=bdm,
                            instance_type=instance_type)

    # and get our instance_id
    instance = reservation.instances[0]

    #  and loop and wait until ssh is available
    while instance.state == u'pending':
        if log:
            log_yellow("Instance state: %s" % instance.state)
        sleep(10)
        instance.update()
    if log:
        log_green("Instance state: %s" % instance.state)
    if wait_for_ssh_available:
        wait_for_ssh(instance.public_dns_name)

    # update the EBS volumes to be deleted on instance termination
    if delete_on_termination:
        for dev, bd in instance.block_device_mapping.items():
            instance.modify_attribute('BlockDeviceMapping',
                                      ["%s=%d" % (dev, 1)])

    # add a tag to our instance
    connection.create_tags([instance.id], tags)

    if log:
        log_green("Public dns: %s" % instance.public_dns_name)

    # returns our new instance
    return instance


def destroy_ebs_volume(connection, region, volume_id, log=False):
    """ destroys an ebs volume """

    if ebs_volume_exists(connection, region, volume_id):
        if log:
            log_yellow('destroying EBS volume ...')
        try:
            connection.delete_volume(volume_id)
        except:
            # our EBS volume may be gone, but AWS info tables are stale
            # wait a bit and ask again
            sleep(5)
            if not ebs_volume_exists(connection, region, volume_id):
                pass
            else:
                raise("Couldn't delete EBS volume")


def destroy_ec2(connection, region, instance_id, log=False):
    """ terminates the instance """

    data = get_ec2_info(connection=connection,
                        instance_id=instance_id,
                        region=region)

    instance = connection.terminate_instances(instance_ids=[data['id']])[0]
    if log:
        log_yellow('destroying instance ...')
    while instance.state != "terminated":
        if log:
            log_yellow("Instance state: %s" % instance.state)
        sleep(10)
        instance.update()
    volume_id = data['volume']
    if volume_id:
        destroy_ebs_volume(connection, region, volume_id)


def down_ec2(connection, instance_id, region, log=False):
    """ shutdown of an existing EC2 instance """
    # get the instance_id from the state file, and stop the instance
    instance = connection.stop_instances(instance_ids=instance_id)[0]
    while instance.state != "stopped":
        if log:
            log_yellow("Instance state: %s" % instance.state)
        sleep(10)
        instance.update()
    if log:
        log_green('Instance state: %s' % instance.state)


def ebs_volume_exists(connection, region, volume_id):
    """ finds out if a ebs volume exists """
    for vol in connection.get_all_volumes():
        if vol.id == volume_id:
            return True
    return False


def ec2():
    env.cloud = 'ec2'


def get_ec2_info(connection,
                 instance_id,
                 region,
                 username=None):
    """ queries EC2 for details about a particular instance_id
    """
    instance = connection.get_only_instances(
        filters={'instance_id': instance_id}
        )[0]

    data = instance.__dict__
    data['state'] = instance.state
    data['cloud_type'] = 'ec2'

    try:
        volume = connection.get_all_volumes(
            filters={'attachment.instance-id': instance.id}
        )[0].id
        data['volume'] = volume
    except:
        data['volume'] = ''
    return data


def up_ec2(connection,
           region,
           instance_id,
           wait_for_ssh_available=True,
           log=False,
           timeout=600):
    """ boots an existing ec2_instance """

    # boot the ec2 instance
    instance = connection.start_instances(instance_ids=instance_id)[0]
    instance.update()
    while instance.state != "running" and timeout > 1:
        log_yellow("Instance state: %s" % instance.state)
        if log:
            log_yellow("Instance state: %s" % instance.state)
        sleep(10)
        timeout = timeout - 10
        instance.update()

    # and make sure we don't return until the instance is fully up
    if wait_for_ssh_available:
        wait_for_ssh(instance.ip_address)



from zope.interface import Interface, Attribute
from flufl.enum import Enum


class Distribution(Enum):
    """
    Enumeration of distributions supported by the bookshelf v3 api.

    :ivar CENTOS7: constant for CentOS 7.
    :ivar UBUNTU1404: constante for Ubuntu LTS 14.04.
    """
    CENTOS7 = u"centos7"
    UBUNTU1404 = u"ubuntu1404"


class ICloudInstanceFactory(Interface):
    """
    Interface for an object that can create cloud instances either
    from some existing serialized/saved state, or create a new cloud
    instance from a configuration.
    """

    def create_from_config(config, distro, region):
        """
        Creates a new instance of the specified distro from the given
        configuration.

        :param dict config: An implementation-specific configuration
            dictionary. This is most likely something read in from a
            configuration language and passed directly to this layer.
        :param Distribution distro: The distribution to spin the instance up
            as.
        :param unicode region: The region to spin the instance up within.

        :return: An :class:`ICloudInstance` provider for a newly created
            instance with type distro.
        """
        pass

    def create_from_saved_state(config, saved_state):
        """
        Re-create or connect to an existing cloud instance as specified in some
        saved state and configuration.

        :param config: An opaque dict of configuration that might be
            specific to a given implementation
        :param saved_state: The serialized state of an instance that
            has most likely been loaded from disk (e.g. a json file).
            This saved state will be used to reconnect to an existing
            instance.

        :return: An :class:`ICloudInstance` provider loaded from the
            saved_state.
        """
        pass


class ICloudInstance(Interface):
    """
    Interface for interacting with a single cloud interface.
    """
    cloud_type = Attribute(
        """The name of the cloud this instance comes from.""")

    username = Attribute(
        """The username to use to log into the instance.""")

    key_filename = Attribute(
        """The filename of the private key to use to log into the instance.""")

    ip_address = Attribute(
        """Externally accessable IP address for the instance""")

    distro = Attribute(
        """The distribution on the instance. Should be one of the above """
        """Distributions.""")

    region = Attribute("""The region the instance is in.""")

    image_basename = Attribute(
        "The basename for the image. The final name will look like"
        "image_basename-YYYYMMDDHHMMSS")

    def create_image(image_name):
        """
        Creates an image from the boot disk of the instance, and leaves the
        instance in an up (booted) state.

        :param unicode image_name: The name of the image to create.

        :returns: The unique identifier of the image.
        """

    def delete_image(image_name):
        """
        Hack: This should not go in the instance as it's a general cloud
        operation. If more general cloud operaions are needed, they
        should be broken out of the Instance object into their own
        thing.

        Deletes an image from the cloud

        :param unicode image_name: the name of the image to delete
        """

    def destroy():
        """
        Downs and Destroys the instance.
        """

    def down():
        """
        Stops a running instance. Throws an exception if the instance is not
        running. Note that this must be done in a way where the instance can be
        started again.
        """

    def get_state():
        """
        Serializes this instance to a dictionary that can be passed to
        ``ICloudInstanceFactory.create_from_saved_state`` in order to
        re-create an :class:`ICloudInstance` that references the same
        underlying instance.

        :returns dict: A simple dictionary of plain old data that can be
            serialized using the JSON library.
        """
